import os
import base64
from phcli.ph_logs.ph_logs import phlogger
from phcli import define_value as dv
from phcli.ph_aws.ph_sts import PhSts
from phcli.ph_aws.ph_cloud_watch import PhCloudWatch, PhMetricData, PhMetricDataUnit


class PhMetric(object):
    def __init__(self):
        phsts = PhSts().assume_role(
            base64.b64decode(dv.ASSUME_ROLE_ARN).decode(),
            dv.ASSUME_ROLE_EXTERNAL_ID,
        )
        self.phcw = PhCloudWatch(phsts=phsts)
        self.cw_namespace = "Ph/Phcli"

    def _verify_call_type(self, call_type):
        return call_type if call_type in ["cli", "api"] else "unknown"

    def _format_dimensions(self, call_type, api, dag_name, job_name):
        return {
            "User": os.getenv("USER", "unknown"),
            "Type": call_type,
            "Group": 'maxauto',
            "API": api,
            "DAG_NAME": dag_name if dag_name else "debug",
            "JOB_NAME": job_name if job_name else "debug",
        }

    def _put_call_count(self, call_type, api, dag_name, job_name, call_state):
        phlogger.debug("put_metric_data >>> " + " ".join([call_type, api, dag_name, job_name, call_state]))

        call_type = self._verify_call_type(call_type)
        dimensions = self._format_dimensions(call_type, api, dag_name, job_name)

        return self.phcw.put_metric_data(
            self.cw_namespace,
            PhMetricData("TotalCall", dimensions, value=1.0, unit=PhMetricDataUnit.COUNT),
            PhMetricData("SuccessCall", dimensions, value=1.0 if call_state == "success" else 0.0, unit=PhMetricDataUnit.COUNT),
            PhMetricData("FailedCall", dimensions, value=0.0 if call_state == "success" else -1.0, unit=PhMetricDataUnit.COUNT),
        )

    def put_call_cli_count(self, api, dag_name="", job_name="", call_state="success"):
        return self._put_call_count("cli", api, dag_name, job_name, call_state)

    def put_call_api_count(self, api, dag_name, job_name, call_state="success"):
        return self._put_call_count("api", api, dag_name, job_name, call_state)

    def _put_call_duration_time(self, call_type, api, dag_name, job_name, call_state, duration_time: float):
        phlogger.debug("put_metric_data >>> " + " ".join([call_type, api, dag_name, job_name, call_state, str(duration_time)]))

        call_type = self._verify_call_type(call_type)
        dimensions = self._format_dimensions(call_type, api, dag_name, job_name)

        return self.phcw.put_metric_data(
            self.cw_namespace,
            PhMetricData("Duration", dimensions, value=duration_time if call_state == "success" else -1.0, unit=PhMetricDataUnit.MCS),
        )

    def put_call_cli_duration_time(self, api, dag_name="", job_name="", call_state="success", duration_time: float=0.0):
        return self._put_call_duration_time("cli", api, dag_name, job_name, call_state, duration_time)

    def put_call_api_duration_time(self, api, dag_name, job_name, call_state="success", duration_time: float=0.0):
        return self._put_call_duration_time("api", api, dag_name, job_name, call_state, duration_time)


if __name__ == '__main__':
    import datetime
    starttime = datetime.datetime.now()

    metric = PhMetric()
    metric.put_call_cli_count("maxauto.delete", call_state="failed")
    metric.put_call_cli_count("maxauto.create", call_state="success")
    metric.put_call_api_count("maxauto.delete", dag_name="test_dag", job_name="test_job", call_state="failed")
    metric.put_call_api_count("maxauto.create", dag_name="test_dag", job_name="test_job", call_state="success")

    metric.put_call_cli_duration_time("maxauto.create", call_state="success", duration_time=(datetime.datetime.now()-starttime).microseconds)



import os


def get_spark_session_func(job_id):
    def get_spark_sessioin(config = None):
        if not config:
            config = {}

        from pyspark.sql import SparkSession
        os.environ["PYSPARK_PYTHON"] = "python3"
        spark = SparkSession.builder \
            .master("yarn") \
            .appName(str(job_id))
        default_config = {
            'spark.sql.codegen.wholeStage': False,
            "spark.sql.execution.arrow.pyspark.enabled": "true",
        }
        default_config.update(config)
        for k, v in default_config.items():
            spark = spark.config(k, v)
        spark = spark.enableHiveSupport() \
            .getOrCreate()

        access_key = os.getenv("AWS_ACCESS_KEY_ID")
        secret_key = os.getenv("AWS_SECRET_ACCESS_KEY")
        if access_key is not None:
            spark._jsc.hadoopConfiguration().set("fs.s3a.access.key", access_key)
            spark._jsc.hadoopConfiguration().set("fs.s3a.secret.key", secret_key)
            spark._jsc.hadoopConfiguration().set("com.amazonaws.services.s3.enableV4", "true")
            spark._jsc.hadoopConfiguration().set("fs.s3a.impl","org.apache.hadoop.fs.s3a.S3AFileSystem")
            # spark._jsc.hadoopConfiguration().set("fs.s3a.aws.credentials.provider","org.apache.hadoop.fs.s3a.BasicAWSCredentialsProvider")
            spark._jsc.hadoopConfiguration().set("fs.s3a.endpoint", "s3.cn-northwest-1.amazonaws.com.cn")
        return spark
    return get_spark_sessioin



# -*- coding: utf-8 -*-

import re
from phcli.ph_data_clean.clean.common_func import *
from phcli.ph_data_clean.clean.data_clean import DataClean


def pack_qty_unit(final_data_pack_unit):
    """
    将包装单位（数字+单位）转化成纯数字的价格转换比（pack_qty）

    :param final_data_pack_unit: 清洗后的包装单位数值

    :return: pack_qty_int:纯数字价格转换比
    :return: pack_unit_str:纯单位

    """
    pack_qty = re.findall(r"\d+", final_data_pack_unit)
    try:
        pack_qty_int = int(pack_qty[0])
        pack_unit_str = final_data_pack_unit.replace(pack_qty[0], "", 1)
        return pack_qty_int, pack_unit_str
    except IndexError:
        return 0, final_data_pack_unit


class ChcDataClean(DataClean):
    """
    CHC 源数据的清洗规则
    """

    def change_pack_qty_unit(self, *args, **kwargs):
        final_data = kwargs['prev']
        if final_data:
            pack_qty = final_data['PACK_QTY']
            pack_unit = final_data['PACK_UNIT']

            if not pack_unit:
                pass
            elif pack_qty and pack_unit:
                pack_unit = pack_qty_unit(final_data_pack_unit=pack_unit)[1]
            elif not pack_qty and pack_unit:
                pack_qty, pack_unit = pack_qty_unit(final_data_pack_unit=pack_unit)

            final_data['PACK_QTY'] = pack_qty
            final_data['PACK_UNIT'] = pack_unit

        return final_data

    def check_hosp_name_code(self, *args, **kwargs):
        final_data = kwargs['prev']
        if final_data:
            if not final_data['HOSP_CODE'] and final_data['HOSP_NAME']:
                final_data['HOSP_CODE'] = "无"
            elif not final_data['HOSP_NAME'] and final_data['HOSP_CODE']:
                final_data['HOSP_NAME'] = "无"

        return final_data

    def split_spec_pack_unit(self, *args, **kwargs):
        """
        将规格列根据*拆开，前面的写入spec，后面的内容写入pack_unit
        """
        final_data = kwargs['prev']
        if final_data and final_data['SPEC'] \
                and not final_data['PACK_UNIT']\
                and not final_data['PACK_QTY']:
            split_result = final_data['SPEC'].split("*")
            if len(split_result) == 2:
                final_data['PACK_UNIT'] = split_result[1]
                final_data['SPEC'] = split_result[0]

        return final_data

    process = [
        check_format,
        change_key,
        change_year_month,
        change_sales_tag,
        split_spec_pack_unit,  # chc 特有
        change_pack_qty_unit,  # chc 特有
        check_hosp_name_code,  # chc 特有
        reformat_int,
        define_tag_err,
    ]



import os
import pytest
from phcli.ph_lmd.model.ph_layer import PhLayer


@pytest.mark.skip(reason='skip')
def test_ph_layer_package_python():
    args = {
        "runtime": "python",
        "package_name": "test_ph_layer_package_python.zip",
        "lib_path": ".venv/lib/python3.8/site-packages/",
    }

    PhLayer().package(args)
    assert os.path.exists(args["package_name"])
    os.remove("test_ph_layer_package_python.zip")


@pytest.mark.skip(reason='skip')
def test_ph_layer_package_nodejs():
    args = {
        "runtime": "nodejs",
        "package_name": "test_ph_layer_package_nodejs.zip",
    }

    PhLayer().package(args)
    assert os.path.exists(args["package_name"])
    os.remove("test_ph_layer_package_nodejs.zip")


@pytest.mark.skip(reason='Deprecation')
def test_ph_layer_create_local():
    args = {
        "name": "test_ph_layer_create",
        "version": "",
        "layer_path": "file/ph_lmd/python-lambda-example-layer.zip",
        "runtime": "python3.8,python3.6",
    }
    assert PhLayer().create(args) != {}


def test_ph_layer_create_s3():
    args = {
        "name": "test_ph_layer_create",
        "version": "",
        "layer_path": "s3://ph-platform/2020-08-10/layers/python/test_ph_layer_create/python-lambda-example-layer.zip",
        "runtime": 'python3.8,python3.6',
    }
    assert PhLayer().create(args) != {}


def test_ph_layer_lists():
    args = {
        "runtime": "python3.8"
    }
    assert PhLayer().lists(args) != {}


def test_ph_layer_get():
    args = {
        "name": "test_ph_layer_create",
    }
    assert PhLayer().get(args) != {}


@pytest.mark.skip(reason='Used for test_ph_lambda Test')
def test_ph_layer_delete():
    args = {
        "name": "test_ph_layer_create",
    }
    args["version"] = PhLayer().get(args)["LayerVersions"][0]["Version"]
    assert PhLayer().delete(args) != {}



# -*- coding: utf-8 -*-
import os

CLI_VERSION = "2020-08-10"

ASSUME_ROLE_ARN = 'YXJuOmF3cy1jbjppYW06OjQ0NDYwMzgwMzkwNDpyb2xlL1BoLUNsaS1MbWQ='
ASSUME_ROLE_EXTERNAL_ID = "Ph-Cli-Lmd"

BUCKET = "ph-platform"


def createDownLoadPath(local_storage, path):
    return local_storage.createDir((os.path.abspath('') + "/" + path).replace("//", "/"))


UPLOADPATH = CLI_VERSION + "/backups"

# HDFSPATH = ["hdfs://backup:8020/common/public/cpa/0.0.4/YEAR=2015.0/MONTH=2.0"]



import base64
from sqlalchemy import create_engine
from sqlalchemy.orm import sessionmaker
from phcli.ph_db.ph_db_api import PhDBAPI


class PhMysql(PhDBAPI):
    def __init__(self, host, port, user, passwd, db):
        self.engine = None
        self.session = None

        self.host = host
        self.port = port
        self.user = user
        self.passwd = passwd
        self.db = db

        self.engine = create_engine('mysql+pymysql://{user}:{passwd}@{host}:{port}/{db}'.format(**self.__dict__))
        Session = sessionmaker(bind=self.engine)
        self.session = Session()


if __name__ == '__main__':
    ms = PhMysql(
        host=base64.b64decode('cGgtZHctaW5zLWNsdXN0ZXIuY2x1c3Rlci1jbmdrMWpldXJtbnYucmRzLmNuLW5vcnRod2VzdC0xLmFtYXpvbmF3cy5jb20uY24K').decode('utf8')[:-1],
        port=base64.b64decode('MzMwNgo=').decode('utf8')[:-1],
        user=base64.b64decode('cGhhcmJlcnMK').decode('utf8')[:-1],
        passwd=base64.b64decode('QWJjZGUxOTYxMjUK').decode('utf8')[:-1],
        db=base64.b64decode('YWlyZmxvdwo=').decode('utf8')[:-1],
    )

    print(ms.tables())




def mapping():
    return [
        {
            "col_name": "COMPANY",
            "col_desc": "数据公司",
            "candidate": [],
            "type": "String",
            "not_null": False,
        },
        {
            "col_name": "SOURCE",
            "col_desc": "数据来源",
            "candidate": [],
            "type": "String",
            "not_null": False,
        },
        {
            "col_name": "TAG",
            "col_desc": "文件标识",
            "candidate": ['_TAG', '_Tag'],
            "type": "String",
            "not_null": True,
        },
        {
            "col_name": "PACK_ID",
            "col_desc": "PACK_ID",
            "candidate": ['PACK_ID'],
            "type": "String",
            "not_null": True,
        },
        {
            "col_name": "MOLE_NAME_EN",
            "col_desc": "分子名英文",
            "candidate": ['MOLE_NAME_EN'],
            "type": "String",
            "not_null": True,
        },
        {
            "col_name": "MOLE_NAME_CH",
            "col_desc": "分子名中文",
            "candidate": ['MOLE_NAME_CH'],
            "type": "String",
            "not_null": True,
        },
        {
            "col_name": "PROD_DESC",
            "col_desc": "PROD_DESC",
            "candidate": ['PROD_DESC'],
            "type": "String",
            "not_null": True,
        },
        {
            "col_name": "PROD_NAME_CH",
            "col_desc": "PROD_NAME_CH",
            "candidate": ['PROD_NAME_CH'],
            "type": "String",
            "not_null": True,
        },
        {
            "col_name": "CORP_NAME_EN",
            "col_desc": "公司名英文",
            "candidate": ['CORP_NAME_EN'],
            "type": "String",
            "not_null": True,
        },
        {
            "col_name": "CORP_NAME_CH",
            "col_desc": "公司名中文",
            "candidate": ['CORP_NAME_CH'],
            "type": "String",
            "not_null": True,
        },
        {
            "col_name": "MNF_NAME_EN",
            "col_desc": "厂家名英文",
            "candidate": ["MNF_NAME_EN"],
            "type": "String",
            "not_null": True,
        },
        {
            "col_name": "MNF_NAME_CH",
            "col_desc": "厂家名中文",
            "candidate": ['MNF_NAME_CH'],
            "type": "Integer",
            "not_null": True,
        },
        {
            "col_name": "PCK_DESC",
            "col_desc": "规格",
            "candidate": ['PCK_DESC'],
            "type": "String",
            "not_null": True,
        },
        {
            "col_name": "DOSAGE",
            "col_desc": "剂型",
            "candidate": ['DOSAGE'],
            "type": "Integer",
            "not_null": True,
        },
        {
            "col_name": "SPEC",
            "col_desc": "规格",
            "candidate": ["SPEC"],
            "type": "String",
            "not_null": True,
        },
        {
            "col_name": "PACK",
            "col_desc": "包装",
            "candidate": ['PACK'],
            "type": "String",
            "not_null": True,
        },
        {
            "col_name": "NFC1",
            "col_desc": "NFC",
            "candidate": ["NFC1"],
            "type": "String",
            "not_null": True,
        },
        {
            "col_name": "NFC1_NAME",
            "col_desc": "NFC1_NAME",
            "candidate": ['NFC1_NAME'],
            "type": "String",
            "not_null": True,
        },
        {
            "col_name": "NFC1_NAME_CH",
            "col_desc": "NFC1_NAME_CH",
            "candidate": ['NFC1_NAME_CH'],
            "type": "String",
            "not_null": True,
        },
        {
            "col_name": "NFC12",
            "col_desc": "NFC12",
            "candidate": ['NFC12'],
            "type": "String",
            "not_null": True,
        },
        {
            "col_name": "NFC12_NAME",
            "col_desc": "NFC12_NAME",
            "candidate": ['NFC12_NAME'],
            "type": "String",
            "not_null": True,
        },
        {
            "col_name": "NFC12_NAME_CH",
            "col_desc": "NFC12_NAME_CH",
            "candidate": ['NFC12_NAME_CH'],
            "type": "String",
            "not_null": True,
        },
        {
            "col_name": "NFC123",
            "col_desc": "NFC123",
            "candidate": ['NFC123'],
            "type": "String",
            "not_null": True,
        },
        {
            "col_name": "NFC123_NAME",
            "col_desc": "NFC123_NAME",
            "candidate": ['NFC123_NAME'],
            "type": "Double",
            "not_null": True,
        },
        {
            "col_name": "CORP_ID",
            "col_desc": "CORP_ID",
            "candidate": ["CORP_ID"],
            "type": "String",
            "not_null": True,
        },
        {
            "col_name": "MNF_TYPE",
            "col_desc": "MNF_TYPE",
            "candidate": ["MNF_TYPE"],
            "type": "String",
            "not_null": True,
        },
        {
            "col_name": "MNF_TYPE_NAME",
            "col_desc": "MNF_TYPE_NAME",
            "candidate": ['MNF_TYPE_NAME'],
            "type": "Double",
            "not_null": True,
        },
        {
            "col_name": "MNF_TYPE_NAME_CH",
            "col_desc": "MNF_TYPE_NAME_CH",
            "candidate": ['MNF_TYPE_NAME_CH'],
            "type": "String",
            "not_null": True,
        },
        {
            "col_name": "MNF_ID",
            "col_desc": "MNF_ID",
            "candidate": ['MNF_ID'],
            "type": "String",
            "not_null": True,
        },
        {
            "col_name": "ATC1_CODE",
            "col_desc": "ATC1_CODE",
            "candidate": ["ATC1_CODE"],
            "type": "String",
            "not_null": True,
        },
        {
            "col_name": "ATC1_DESC",
            "col_desc": "ATC1_DESC",
            "candidate": ["ATC1_DESC"],
            "type": "String",
            "not_null": True,
        },
        {
            "col_name": "ATC2_CODE",
            "col_desc": "ATC2_CODE",
            "candidate": ["ATC2_CODE"],
            "type": "String",
            "not_null": True,
        },
        {
            "col_name": "ATC2_DESC",
            "col_desc": "ATC2_DESC",
            "candidate": ["ATC2_DESC"],
            "type": "String",
            "not_null": True,
        },
        {
            "col_name": "ATC3_CODE",
            "col_desc": "ATC3_CODE",
            "candidate": ["ATC3_CODE"],
            "type": "String",
            "not_null": True,
        },
        {
            "col_name": "ATC3_DESC",
            "col_desc": "ATC3_DESC",
            "candidate": ["ATC3_DESC"],
            "type": "String",
            "not_null": True,
        },
        {
            "col_name": "ATC4_CODE",
            "col_desc": "ATC4_CODE",
            "candidate": ["ATC4_CODE"],
            "type": "String",
            "not_null": True,
        },
        {
            "col_name": "ATC4_DESC",
            "col_desc": "ATC4_DESC",
            "candidate": ["ATC4_DESC"],
            "type": "String",
            "not_null": True,
        },
        {
            "col_name": "ATC1",
            "col_desc": "ATC1",
            "candidate": ["ATC1"],
            "type": "String",
            "not_null": True,
        },
        {
            "col_name": "ATC2",
            "col_desc": "ATC2",
            "candidate": ["ATC2"],
            "type": "String",
            "not_null": True,
        },
        {
            "col_name": "ATC3",
            "col_desc": "ATC3",
            "candidate": ["ATC3"],
            "type": "String",
            "not_null": True,
        },
        {
            "col_name": "ATC4",
            "col_desc": "ATC4",
            "candidate": ["ATC4"],
            "type": "String",
            "not_null": True,
        },
        {
            "col_name": "REMARK",
            "col_desc": "REMARK",
            "candidate": ["REMARK"],
            "type": "String",
            "not_null": False,
        },
    ]



# -*- coding: utf-8 -*-

import os
import sys
import string
import boto3

from phcli.ph_errs.ph_err import PhException
from phcli.ph_aws.aws_root import PhAWS


class PhS3(PhAWS):
    def __init__(self, *args, **kwargs):
        self.access_key = kwargs.get('access_key', None)
        self.secret_key = kwargs.get('secret_key', None)
        if self.access_key and self.secret_key:
            self.s3_client = boto3.client('s3', region_name='cn-northwest-1',
                                          aws_access_key_id=self.access_key,
                                          aws_secret_access_key=self.secret_key)
            self.s3_resource = boto3.resource('s3', region_name='cn-northwest-1',
                                              aws_access_key_id=self.access_key,
                                              aws_secret_access_key=self.secret_key)
            return

        self.phsts = kwargs.get('phsts', None)
        if self.phsts and self.phsts.credentials:
            self.s3_client = boto3.client('s3', **self.phsts.get_cred())
            self.s3_resource = boto3.resource('s3', **self.phsts.get_cred())
            return

        self.s3_client = boto3.client('s3')
        self.s3_resource = boto3.resource('s3')

    def list_buckets(self):
        bks = self.s3_client.list_buckets()["Buckets"]
        bks_names = []
        for it in enumerate(bks):
            bks_names.append(it[1]["Name"])
        return bks_names

    def upload(self, file, bucket_name, object_name):
        """
        上传本地文件到 S3
        :param file: 本地文件路径
        :param bucket_name: S3 桶名字
        :param object_name: S3 文件路径
        :return:
        """
        self.s3_client.upload_file(
            Bucket=bucket_name,
            Key=object_name,
            Filename=file
        )

    def upload_dir(self, dir, bucket_name, s3_dir):
        """
        上传本地目录到 S3
        :param dir: 本地目录路径
        :param bucket_name: S3 桶名字
        :param s3_dir: S3 目录路径
        :return:
        """
        dir = dir if dir.endswith("/") else dir+"/"
        s3_dir = s3_dir if s3_dir.endswith("/") else s3_dir+"/"

        for key in os.listdir(dir):
            if os.path.isfile(dir+key):
                self.upload(dir+key, bucket_name, s3_dir+key)
            else:
                self.upload_dir(dir+key, bucket_name, s3_dir+key)

    def delete(self, bucket_name, object_name):
        """
        删除 s3 上的一个文件
        :param bucket_name: 桶名
        :param object_name: S3 文件路径
        :return:
        """
        self.s3_client.delete_object(
            Bucket=bucket_name,
            Key=object_name,
        )

    def delete_dir(self, bucket_name, s3_dir):
        """
        删除 s3 上的一个目录
        :param bucket_name: S3 桶名字
        :param s3_dir: S3 目录路径
        :return:
        """
        bucket = self.s3_resource.Bucket(bucket_name)
        bucket.objects.filter(Prefix=s3_dir).delete()

    def open_object(self, bucket_name, object_name):
        """
        使用字符串打开 S3 文件
        :param bucket_name: S3 桶名字
        :param object_name: S3 文件路径
        :return: str
        """
        response = self.s3_client.get_object(
            Bucket=bucket_name,
            Key=object_name
        )
        if sys.version_info > (3, 0):
            return response["Body"].read().decode()
        else:
            return response["Body"].read()

    def open_object_by_lines(self, bucket_name, object_name):
        """
        按行打开 S3 文件
        :param bucket_name: S3 桶名字
        :param object_name: S3 文件路径
        :return: List[str]
        """
        object_str = self.open_object(bucket_name, object_name)
        if sys.version_info > (3, 0):
            return str.split(object_str, "\n")
        else:
            return string.split(object_str, "\n")

    def download(self, bucket_name, object_name, file):
        """
        下载 S3 文件
        :param bucket_name: S3 桶名字
        :param object_name: S3 文件路径
        :param file: 本地文件路径
        :return:
        """
        f = open(file, "w")
        for line in self.open_object_by_lines(bucket_name, object_name):
            f.write(line + "\n")
        f.close()

    def __url_get_bucket_info(self, path):
        """
        根据 S3 URL 分析出 S3 Bucket 和文件的具体路径
        :param path: S3 URL
        :return: [bucket_name, file_path]
        """
        if not isinstance(path, str):
            raise PhException('Expected an str')

        if path.startswith("https://") or path.startswith("http://"):
            url = path.split("://")[1]
            bucket_name = url.split(".")[0]
            file_path = url.split(".amazonaws.com.cn/")[1]
            return [bucket_name, file_path]
        elif path.startswith("s3://"):
            url = path.split("://")[1]
            bucket_name = url.split("/")[0]
            file_path = "/".join(url.split("/")[1:])
            return [bucket_name, file_path]
        else:
            raise PhException("The url is wrong")

    def sync_file_local_to_s3(self, path, bucket_name, dir_name, version=''):
        """
        如果参数的是本地文件，则自动同步到 S3, 然后返回桶名和文件路径
        如果参数的是 S3 的文件，则直接返回桶名和文件路径
        :param path: 文件路径
        :param bucket_name: 同步的桶名称
        :param dir_name: 放置到 S3 的目录位置
        :param version: 文件版本
        :param credentials: assumerole 证书，没有则使用执行者证书
        :return: [bucket_name, file_path]
        """

        if path.startswith("https://") or path.startswith("http://") or path.startswith("s3://"):
            return self.__url_get_bucket_info(path)
        else:
            object_name = path.split("/")[-1]
            if version != "":
                object_name = ".".join(object_name.split(".")[:-1]) + "-" + version + "." + object_name.split(".")[-1]

            if dir_name.endswith("/"):
                object_name = dir_name + object_name
            else:
                object_name = dir_name + "/" + object_name

            self.upload(path, bucket_name, object_name)
            return [bucket_name, object_name]

    def get_bucket_files_path(self, bucket_name, path_prefix):
        """
            获取 S3 指定桶下指定路径下所有文件的路径
            :param bucket_name: 桶名
            :param path_prefix: S3 文件路径
        """
        file_paths = []
        res = self.s3_client.list_objects(
            Bucket=bucket_name,
            Prefix=path_prefix
        )
        if 'Contents' in res:
            for item in res['Contents']:
                file_paths.append(item['Key'])
        return file_paths

    def copy_file(self, source_bucket, source_file_path, target_bucket, target_file_path):
        """
            复制 S3 指定资源桶指定资源路径的所有内容到目标桶目标路径下
            :param source_bucket: 资源桶名
            :param source_file_path: 资源路径
            :param target_bucket: 目标桶
            :param target_file_path: 目标路径
        """
        copy_source = {
            'Bucket': source_bucket,
            'Key': source_file_path
        }
        self.s3_resource.meta.client.copy(copy_source, target_bucket, target_file_path)



import time
from phcli.ph_max_auto import define_value as dv


def get_run_time():
    return time.strftime("%Y-%m-%d_%H:%M:%S", time.localtime())


def get_dag_name(kwargs):
    return kwargs.get("dag_name", "dag_name" + "hbzbao_dag_test")


def get_run_id(kwargs):
    run_id = kwargs["run_id"]
    if not run_id:
        run_id = "runid_" + "alfred_runner_test"
    return run_id


def get_job_full_name(kwargs):
    job_full_name = kwargs.get("job_full_name", "job_full_name" + "hbzhao_job_full_name_test")
    return job_full_name


def get_job_name(kwargs):
    job_name = kwargs.get("name", "name" + "hbzhao_name_test")
    return job_name


def get_job_id(kwargs, job_name):
    g_flag = kwargs.get("g_flag", "")
    job_id = job_name + g_flag
    return job_id


def get_result_path(kwargs, job_name=None):
    run_id = get_run_id(kwargs)
    if not job_name:
        job_name = get_job_name(kwargs)
    if 'path_prefix' in kwargs:
        path_prefix = kwargs['path_prefix']
    else:
        path_suffix = kwargs.get('path_suffix', dv.DEFAULT_RESULT_PATH_SUFFIX)
        path_prefix = dv.DEFAULT_RESULT_PATH_FORMAT_STR.format(
            bucket_name=dv.DEFAULT_RESULT_PATH_BUCKET,
            version=dv.DEFAULT_RESULT_PATH_VERSION,
            dag_name=get_dag_name(kwargs),
        ) + path_suffix
    return path_prefix + "/" + run_id + "/" + job_name + "/"


def get_depends_file_path(kwargs, job_name, job_key):
    return get_result_path(kwargs, job_name) + job_key


def get_depends_path(kwargs):
    depends_lst = eval(kwargs.get("depend_job_names_keys", "[]"))
    result = {}
    for item in depends_lst:
        tmp_lst = item.split("#")
        depends_job = tmp_lst[0]
        depends_key = tmp_lst[1]
        depends_name = tmp_lst[2]
        result[depends_name] = get_depends_file_path(kwargs, depends_job, depends_key)
    return result


def get_asset_path_prefix(kwargs):
    path_suffix = kwargs.get('asset_path_suffix', dv.DEFAULT_ASSET_PATH_SUFFIX)
    asset_path_prefix = dv.DEFAULT_ASSET_PATH_FORMAT_STR.format(
            bucket_name=dv.DEFAULT_ASSET_PATH_BUCKET,
            version=dv.DEFAULT_ASSET_PATH_VERSION,
        ) + path_suffix
    return asset_path_prefix + '/'


if __name__ == '__main__':
    path_prefix_suffix_result = get_result_path({
        "name": "test_job",
        "dag_name": "test_dag",
        "run_id": "test_run_id",
        "g_flag": "12"
    })
    print("path_prefix_suffix_result = " + path_prefix_suffix_result)

    # assert(path_prefix_suffix_result == "s3a://ph-max-auto/2020-08-11/test_dag/refactor/runs/runid_alfred_runner_test/test_job/")
    #
    # path_prefixexist_suffixexist_result = get_result_path({
    #     "name": "test_job",
    #     "dag_name": "test_dag",
    #     "path_prefix": "s3a://exist",
    #     "path_suffix": "exist"
    # })
    # print("path_prefixexist_suffixexist_result = " + path_prefixexist_suffixexist_result)
    # assert(path_prefixexist_suffixexist_result == "s3a://exist/runid_alfred_runner_test/test_job/")
    #
    path_prefixexist_suffixexist_depends_file_path = get_depends_file_path({
        "name": "test_job",
        "dag_name": "test_dag",
        "run_id": "test_run_id",
        "g_flag": "12"
    }, "test_job", "test_key")
    print("path_prefixexist_suffixexist_depends_file_path = " + path_prefixexist_suffixexist_depends_file_path)
    # assert(path_prefixexist_suffixexist_depends_file_path == "s3a://exist/runid_alfred_runner_test/test_job/test1/c")
    #
    # prefixexist_suffixexist_depends_path = get_depends_path({
    #     "name": "test_job",
    #     "dag_name": "test_dag",
    #     "path_prefix": "s3a://exist",
    #     "path_suffix": "exist",
    #     # "depend_job_names_keys": '["effectiveness_adjust_mnf#mnf_adjust_result#mnf_adjust", "effectiveness_adjust_spec#spec_adjust_result#spec_adjust"]'
    # })
    # print("prefixexist_suffixexist_depends_path = " + str(prefixexist_suffixexist_depends_path))
    # assert len(prefixexist_suffixexist_depends_path) == 2
    # assert "mnf_adjust" in prefixexist_suffixexist_depends_path
    # assert prefixexist_suffixexist_depends_path['mnf_adjust'] == 's3a://exist/runid_alfred_runner_test/test_job/effectiveness_adjust_mnf/mnf_adjust_result'
    # assert "spec_adjust" in prefixexist_suffixexist_depends_path
    # assert prefixexist_suffixexist_depends_path['spec_adjust'] == 's3a://exist/runid_alfred_runner_test/test_job/effectiveness_adjust_spec/spec_adjust_result'
    #
    # test_asset_path = get_asset_path({
    #     'name': 'test_job',
    #     'dag_name': 'test_dag'
    # })
    # print("test_asset_path = " + test_asset_path)



# -*- coding: utf-8 -*-

import boto3
import base64
import time

from phcli.ph_aws.ph_sts import PhSts
from phcli.ph_aws.ph_s3 import PhS3
from phcli.ph_lmd.model.aws_operator import AWSOperator
from phcli.ph_lmd.model.ph_layer import PhLayer
from phcli.ph_lmd import define_value as dv
from phcli.ph_lmd.runtime.rt_util import get_short_rt, get_rt_inst


class PhLambda(AWSOperator):
    """
    lambda 的源代码
    """

    phsts = PhSts().assume_role(
        base64.b64decode(dv.ASSUME_ROLE_ARN).decode(),
        dv.ASSUME_ROLE_EXTERNAL_ID,
    )
    phs3 = PhS3(phsts=phsts)

    def __sync_file(self, data):
        bucket_name, object_name = self.phs3.sync_file_local_to_s3(
            data["lambda_path"],
            bucket_name=data.get("bucket", dv.DEFAULT_BUCKET),
            dir_name=dv.CLI_VERSION + dv.DEFAULT_LAMBDA_DIR
                .replace("#runtime#", get_short_rt(data["runtime"]))
                .replace("#name#", data["name"]),
            version=data.get("version", ""),
        )
        return bucket_name, object_name

    def __create_func(self, data, lambda_client):
        bucket_name, object_name = self.__sync_file(data)

        role_arn = base64.b64decode(dv.ASSUME_ROLE_ARN).decode()

        vpc_config = data['vpc_config'] if 'vpc_config' in data.keys() else {}

        layers_arn = []
        ph_layer = PhLayer()
        for layer_name in data["lambda_layers"].split(","):
            layers_arn.append(ph_layer.get({"name": layer_name})["LayerVersions"][0]["LayerVersionArn"])

        lambda_response = lambda_client.create_function(
            FunctionName=data["name"],
            Runtime=data["runtime"].split(",")[0],
            Role=role_arn,
            Handler=data["lambda_handler"],
            Code={
                'S3Bucket': bucket_name,
                'S3Key': object_name,
            },
            Description=data.get("lambda_desc", "phcli create " + data['name'] + " lambda function"),
            Timeout=data.get("lambda_timeout", 30),
            MemorySize=data.get("lambda_memory_size", 128),
            # Publish=True|False,
            VpcConfig=vpc_config,
            # DeadLetterConfig={
            #     'TargetArn': 'string'
            # },
            Environment={
                'Variables': data.get("lambda_env", {}),
            },
            # KMSKeyArn='string',
            # TracingConfig={
            #     'Mode': 'Active'|'PassThrough'
            # },
            Tags=data.get("lambda_tag", {}),
            Layers=layers_arn,
        )
        return lambda_response

    def __apply_version(self, data, lambda_client):
        def create_version():
            response = lambda_client.publish_version(
                FunctionName=data["name"],
                # CodeSha256='string',
                # Description='string',
                # RevisionId='string'
            )
            return response

        return create_version()

    def __apply_alias(self, data, lambda_client, create_version_response):

        def create_alias(name, function_version):
            response = lambda_client.create_alias(
                FunctionName=data["name"],
                Name=name,
                FunctionVersion=function_version,
                Description=data.get("lambda_desc", "phcli create " + data['name'] + " lambda function alias"),
                # RoutingConfig={
                #     'AdditionalVersionWeights': {
                #         'string': 123.0
                #     }
                # }
            )
            return response

        def delete_alias(name):
            response = lambda_client.delete_alias(
                FunctionName=data["name"],
                Name=name,
            )
            return response

        def cur2prev(aliases):
            if dv.LAMBDA_FUNCTION_ALIAS_NAME_PREV in aliases.keys():
                delete_alias(dv.LAMBDA_FUNCTION_ALIAS_NAME_PREV)

            version = aliases[dv.LAMBDA_FUNCTION_ALIAS_NAME_CUR]['FunctionVersion']
            create_alias(dv.LAMBDA_FUNCTION_ALIAS_NAME_PREV, version)
            return delete_alias(dv.LAMBDA_FUNCTION_ALIAS_NAME_CUR)

        resp = self.get(data)
        aliases = resp['Aliases'] if resp else []
        aliases = dict([(alias['Name'], alias) for alias in aliases])

        # 如果版本没变化
        if aliases and aliases[dv.LAMBDA_FUNCTION_ALIAS_NAME_CUR]['FunctionVersion'] == create_version_response["Version"]:
            return aliases[dv.LAMBDA_FUNCTION_ALIAS_NAME_CUR]['FunctionVersion']

        if aliases:
            cur2prev(aliases)

        return create_alias(dv.LAMBDA_FUNCTION_ALIAS_NAME_CUR, create_version_response["Version"])

    def package(self, data):
        """
        对 lambda 源代码打包
        :param data:
            :arg runtime 运行时字符串，“python” 或者 “nodejs” 或者 “go”
            :arg code_path: lambda 代码位置
            :arg package_name 打包的名称
        """
        runtime_inst = get_rt_inst(data['runtime'])
        return runtime_inst.pkg_code(data)

    def create(self, data):
        """
        创建 lambda, 并发布一个新版本，然后使用 version 定义一个别名
        :param data:
            :arg name: 创建的 lambda 函数的名字
            :arg version: lambda 函数别名版本，默认为 alpha
            :arg runtime: lambda 函数适用的运行时，如果多个请使用 “,” 分割, 但是实际使用以第一个为准
            :arg lambda_path: lambda zip 的位置
                            可以是本地（file/ph_lmd/python-lambda-example-code.zip，会先被传到 S3）或
                            s3 上的文件（s3://ph-platform/2020-08-10/functions/python/test_ph_lambda_create/python-lambda-example-code.zip）
            :arg lambda_handler: lambda 的入口函数
            :arg lambda_layers: lambda 函数依赖的层名称，如果多个请使用 “,” 分割
            :arg lambda_desc: lambda 函数的描述
            :arg lambda_timeout: [int] lambda 的超时时间，默认30s（官方是3s）
            :arg lambda_memory_size: [int] lambda 的使用内存，默认128
            :arg lambda_env: [dict] lambda 的环境变量
            :arg lambda_tag: [dict] lambda 的标签
            :arg vpc_config: [dict] lambda VPC 配置
        """
        lambda_client = boto3.client('lambda', **self.phsts.get_cred())

        self.__create_func(data, lambda_client)

        # 首次创建并且配置 VPC 的情况下，function 会有一段 pending 时间，因此等待 30 s
        if 'vpc_config' in data.keys():
            for i in range(30):
                time.sleep(1)

        response = self.__apply_version(data, lambda_client)

        response = self.__apply_alias(data, lambda_client, response)

        return response

    def lists(self, data):
        """
        获取所有 lambda 实例
        """
        lambda_client = boto3.client('lambda', **self.phsts.get_cred())

        response = lambda_client.list_functions(
            # MasterRegion='string',
            FunctionVersion='ALL',
            # Marker='string',
            MaxItems=50,
        )

        return response

    def get(self, data):
        """
        获取指定 lambda 实例
        :param data:
            :arg name: 要查询的 lambda 函数的名字
        """
        lambda_client = boto3.client('lambda', **self.phsts.get_cred())

        try:
            response = lambda_client.get_function(
                FunctionName=data["name"],
                # Qualifier='string',
            )

            versions = lambda_client.list_versions_by_function(
                FunctionName=data["name"],
                # Marker='string',
                # MaxItems=123
            )["Versions"]
            versions.reverse()
            response["Versions"] = versions

            aliases = lambda_client.list_aliases(
                FunctionName=data["name"],
            )["Aliases"]
            aliases.reverse()
            response["Aliases"] = aliases
        except:
            response = {}

        return response

    def update(self, data):
        """
        更新 lambda, 并发布一个新版本，然后使用 version 定义一个别名
        :param data:
            :arg name: 创建的 lambda 函数的名字 【必需】
            :arg version: lambda 函数别名版本
            :arg runtime: lambda 函数适用的运行时，如果多个请使用 “,” 分割, 但是实际使用以第一个为准
            :arg lambda_path: lambda zip 的位置
                            可以是本地（file/ph_lmd/python-lambda-example-code.zip，会先被传到 S3）或
                            s3 上的文件（s3://ph-platform/2020-08-10/functions/python/test_ph_lambda_create/python-lambda-example-code.zip)
            :arg lambda_handler: lambda 的入口函数
            :arg lambda_layers: lambda 函数依赖的层名称，如果多个请使用 “,” 分割
            :arg lambda_desc: lambda 函数的描述
            :arg lambda_timeout: [int] lambda 的超时时间，默认30s（官方是3s）
            :arg lambda_memory_size: [int] lambda 的使用内存，默认128
            :arg lambda_env: [dict] lambda 的环境变量
            :arg vpc_config: [dict] lambda VPC 配置
        """
        lambda_client = boto3.client('lambda', **self.phsts.get_cred())

        # 更新代码
        if "lambda_path" in data.keys():
            bucket_name, object_name = self.__sync_file(data)

            lambda_client.update_function_code(
                FunctionName=data["name"],
                # ZipFile=b'bytes',
                S3Bucket=bucket_name,
                S3Key=object_name,
                # S3ObjectVersion='string',
                # Publish=True|False,
                # DryRun=True|False,
                # RevisionId='string'
            )
            del data["lambda_path"]
            return self.update(data)

        # 更新配置
        else:
            response = self.get(data)

            role_arn = base64.b64decode(dv.ASSUME_ROLE_ARN).decode()

            layers_arn = []
            if "lambda_layers" in data.keys():
                ph_layer = PhLayer()
                for layer_name in data["lambda_layers"].split(","):
                    layers_arn.append(ph_layer.get({"name": layer_name})["LayerVersions"][0]["LayerVersionArn"])
            else:
                for layer_name in response["Configuration"]["Layers"]:
                    layers_arn.append(layer_name["Arn"])

            vpc_config = data['vpc_config'] if 'vpc_config' in data.keys() else {}

            lambda_response = lambda_client.update_function_configuration(
                FunctionName=data["name"],
                Role=role_arn,
                Handler=data.get("lambda_handler", response["Configuration"]["Handler"]),
                Description=data.get("lambda_desc", response["Configuration"]["Description"]),
                Timeout=data.get("lambda_timeout", response["Configuration"]["Timeout"]),
                MemorySize=data.get("lambda_memory_size", response["Configuration"]["MemorySize"]),
                VpcConfig=vpc_config,
                Environment={
                    'Variables': data.get("lambda_env", response["Configuration"].get("Environment", {}).get("Variables", {})),
                },
                Runtime=data.get("runtime", [response["Configuration"]["Runtime"]]).split(",")[0],
                # DeadLetterConfig={
                #     'TargetArn': 'string'
                # },
                # KMSKeyArn='string',
                # TracingConfig={
                #     'Mode': 'Active'|'PassThrough'
                # },
                # RevisionId='string',
                Layers=layers_arn
            )

        response = self.__apply_version(data, lambda_client)

        response = self.__apply_alias(data, lambda_client, response)

        return response

    def apply(self, data):
        """
        发布或更新 lambda, 并发布一个新版本，然后使用 version 定义一个别名
        :param data:
            :arg name: 创建的 lambda 函数的名字
            :arg version: lambda 函数别名版本
            :arg runtime: lambda 函数适用的运行时，如果多个请使用 “,” 分割, 但是实际使用以第一个为准
            :arg lambda_path: lambda zip 的位置
                            可以是本地（file/ph_lmd/python-lambda-example-code.zip，会先被传到 S3）或
                            s3 上的文件（s3://ph-platform/2020-08-10/functions/python/test_ph_lambda_create/python-lambda-example-code.zip）
            :arg lambda_handler: lambda 的入口函数
            :arg lambda_layers: lambda 函数依赖的层名称，如果多个请使用 “,” 分割
            :arg lambda_desc: lambda 函数的描述
            :arg lambda_timeout: [int] lambda 的超时时间，默认30s（官方是3s）
            :arg lambda_memory_size: [int] lambda 的使用内存，默认128
            :arg lambda_env: [dict] lambda 的环境变量
            :arg lambda_tag: [dict] lambda 的标
        """
        if self.get(data) == {}:
            return self.create(data)
        else:
            return self.update(data)

    def stop(self, data):
        """
        停止 lambda 继续处理请求，并发数设置为 0
        :param data:
            :arg name: 创建的 lambda 函数的名字 【必需】
        """
        data["lambda_concurrent"] = 0
        return self.start(data)

    def start(self, data):
        """
        重新激活 lambda，使其可以接受请求，并发数设置为指定值或默认值
        :param data:
            :arg name: 创建的 lambda 函数的名字 【必需】
            :arg lambda_concurrent: 分配给别名的预留并发值, 不指定则使用非预留账户并发
        """
        lambda_client = boto3.client('lambda', **self.phsts.get_cred())

        if "lambda_concurrent" in data.keys():
            response = lambda_client.put_function_concurrency(
                FunctionName=data["name"],
                ReservedConcurrentExecutions=data["lambda_concurrent"],
            )
        else:
            response = lambda_client.delete_function_concurrency(
                FunctionName=data["name"],
            )

        return response

    def delete(self, data):
        """
        删除 lambda, 版本 或 别名
        :param data:
            :arg name: 创建的 lambda 函数的名字 【必需】
            :arg version: lambda 函数版本, 不传或传 #ALL# 则删除整个 lambda 函数
        """
        lambda_client = boto3.client('lambda', **self.phsts.get_cred())

        if "version" not in data.keys() or data["version"] == "#ALL#":
            response = lambda_client.delete_function(
                FunctionName=data["name"],
            )
        else:
            response = lambda_client.delete_function(
                FunctionName=data["name"],
                Qualifier=data["version"],
            )

        return response





# -*- coding: utf-8 -*-

from phcli.ph_lmd.runtime.lambda_rt import LambdaRuntime


class GoRT(LambdaRuntime):
    pass
    # __runtime_name = "golang"
    #
    # def pkg_layer(self, data):
    #     """
    #     组织 layer 的打包逻辑
    #     :param data
    #         :arg is_pipenv: 是否使用的 pipenv 构建的项目，默认为 False
    #         :arg lib_path: 仅当 is_pipenv = False 时有效，指定 python 的 lib 位置
    #         :arg package_name: 打包的名称
    #     """
    #
    #     self._package_cmds = [
    #         "mkdir -p %s/%s" % (self._package_root, self.__runtime_name),
    #         ]
    #
    #     if data.get("is_pipenv", False):
    #         self._package_cmds.extend([
    #             "pipenv lock -r | grep '==' | awk -F '==' '{print $1}' |" +
    #             "xargs -I '{}' cp -r ${VIRTUAL_ENV}/lib/python3.8/site-packages/{} %s/%s/" % (
    #                 self._package_root, self.__runtime_name),
    #             ])
    #     else:
    #         self._package_cmds.extend([
    #             "cp -r %s/* %s/%s/" % (data["lib_path"], self._package_root, self.__runtime_name),
    #             ])
    #
    #     self._package_cmds.extend([
    #         "cd %s && zip -r -q ../package.zip . && cd -" % self._package_root,
    #         "rm -rf %s" % self._package_root,
    #         ])
    #
    #     self.package(data["package_name"])
    #
    # def pkg_code(self, data):
    #     """
    #     组织 code 的打包逻辑
    #     :param data
    #         :arg code_path: 需要打包的代码位置
    #         :arg package_name: 打包的名称
    #     """
    #
    #     self._package_cmds = [
    #         "zip -r package.zip %s" % data["code_path"],
    #         ]
    #
    #     self.package(data["package_name"])



import os
import pytest
from phcli.ph_lmd.runtime.nodejs_rt import NodejsRT


nodejs_rt = NodejsRT()


@pytest.mark.skip("lib_path not exists")
def test_nodejs_rt_pkg_layer_libpath():
    args = {
        "lib_path": "node_modules",
        "package_name": "test_nodejs_rt_pkg_layer_libpath.zip"
    }

    nodejs_rt.pkg_layer(args)
    assert os.path.exists(args["package_name"])
    os.remove(args["package_name"])


@pytest.mark.skip("code_path not exists")
def test_nodejs_rt_pkg_code():
    package_name = "test_nodejs_rt_pkg_code.zip"
    args = {
        "code_path": "dist/,config/,app.js",
        "package_name": package_name,
    }
    nodejs_rt.pkg_code(args)
    assert os.path.exists(package_name)
    os.remove(package_name)



import os
from .ph_ide_base import PhIDEBase, PhCompleteStrategy


class PhIDEJupyter(PhIDEBase):
    """
    针对 Jupyter 环境的执行策略
    """

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.logger.debug('maxauto PhIDEJupyter init')
        self.logger.debug(self.__dict__)

    def create(self, **kwargs):
        """
        jupyter 的创建过程
        """
        self.logger.debug('maxauto ide=jupyter 的 create 实现')
        self.logger.debug(self.__dict__)

        self.check_path(self.job_path)

        super().create()

    def complete(self, **kwargs):
        """
        jupyter 的补全过程
        """
        self.logger.debug('maxauto ide=jupyter 的 complete 实现')
        self.logger.debug(self.__dict__)

        def single_complete(job_path, exts):
            require_cs = self.choice_complete_strategy(job_path + ".ipynb", job_path)
            if self.strategy == "s2c":
                actual_cs = PhCompleteStrategy.S2C
            elif self.strategy == 'c2s':
                actual_cs = PhCompleteStrategy.C2S
            else:
                actual_cs = require_cs

            if require_cs != actual_cs:
                self.logger.warn("actual [" + str(actual_cs) + "] != require [" + str(require_cs) + "], in " + job_path)

            if actual_cs == PhCompleteStrategy.S2C:
                self.logger.info("S2C: " + job_path)
                runtime_inst = self.table_driver_runtime_inst(self.runtime)(**self.__dict__)
                if not os.path.exists(job_path + ".ipynb"):
                    self.logger.warn("source path is not exists: " + job_path + ".ipynb")
                    return
                runtime_inst.jupyter_to_c9(job_path + ".ipynb", job_path, exts)
            elif actual_cs == PhCompleteStrategy.C2S:
                self.logger.info("C2S: " + job_path)
                runtime_inst = self.table_driver_runtime_inst(self.runtime)(**self.__dict__)
                if not os.path.exists(job_path):
                    self.logger.warn("source path is not exists: " + job_path)
                    return
                runtime_inst.c9_to_jupyter(job_path, job_path + ".ipynb", exts)
            else:
                self.logger.info("KEEP COMPLETE: " + job_path)

        def recursive_complete_path(path):
            def get_group_job_by_path(path):
                if path.endswith('.ipynb'):
                    tmp = path.split("phjobs/")[-1].split("/")
                    group = tmp[0] if len(tmp) == 2 else 'null'
                    job = tmp[-1].split('.')[0]
                else:
                    tmp = path.split("phjobs/")[-1].split("/")[:-1]
                    group = tmp[0] if len(tmp) == 2 else 'null'
                    job = tmp[-1]
                return group, job

            result = {}
            for root, dirs, files in os.walk(path):
                for file in files:
                    group, job = get_group_job_by_path(os.path.join(root, file))
                    if group in result:
                        jobs = result[group]
                        if job not in jobs:
                            jobs.append(job)
                    else:
                        jobs = [job]
                    result[group] = jobs
            return result

        if self.group == "ALL":
            recursive_path = self.get_workspace_dir() + '/' + self.get_current_project_dir() + self.job_prefix
            gj_map = recursive_complete_path(recursive_path)
        elif self.name == "ALL":
            recursive_path = self.get_workspace_dir() + '/' + self.get_current_project_dir() + self.job_prefix + self.group
            gj_map = recursive_complete_path(recursive_path)
        else:
            group = self.group if self.group else 'null'
            gj_map = {group: [self.name]}

        for group in gj_map:
            for job in gj_map[group]:
                project_path = self.get_workspace_dir() + '/' + self.get_current_project_dir()
                job_path = project_path + self.job_prefix + (group + '/' if group != 'null' else '') + job
                single_complete(job_path, {"group": (group if group != 'null' else ''), "ide": self.ide})



import os
import subprocess
from phcli.ph_logs.ph_logs import phlogger
from phcli.ph_max_auto import define_value as dv
from phcli.ph_max_auto.ph_config.phconfig.phconfig import PhYAMLConfig


DIR_NAME = 'preset_write_asset'


def phconf_buf(context):
    buf = context.phs3.open_object(dv.TEMPLATE_BUCKET, dv.CLI_VERSION + dv.TEMPLATE_PHCONF_FILE)
    buf = buf.replace("$name", 'write_asset') \
        .replace("$runtime", 'python3') \
        .replace("$command", 'script') \
        .replace("$timeout", str(10.0)) \
        .replace("$code", "phmain.py") \
        .replace("args:", '') \
        .replace("$input", '') \
        .replace("outputs:", '') \
        .replace("$output", '')
    return buf


def job_conf_args2map(path):
    config = PhYAMLConfig(path)
    config.load_yaml()

    result = {'input': {}, 'output': {}}
    for arg in config.spec.containers.args:
        if arg.value != "":
            result['input'][arg.key] = str(arg.value)

    for output in config.spec.containers.outputs:
        if output.value != "":
            result['output'][output.key] = str(output.value)

    return result


def merge_job_args(m1, m2):
    for k, v in m2.items():
        if k in m1.keys() and v != m1[k]:
            phlogger.warning("{} 在之前的 job 中已经被定义，将使用 {} 覆盖 {}".format(k, v, m1[k]))
        m1[k] = v


def copy_job(context, **kwargs):
    # 0. init data
    config = PhYAMLConfig(context.combine_path, "/phdag.yaml")
    config.load_yaml()

    # 1. /__init.py file
    subprocess.call(["mkdir", context.dag_path + DIR_NAME])
    subprocess.call(["touch", context.dag_path + DIR_NAME + "/__init__.py"])

    # 2. /phjob.py file
    phjobs_lst = []
    all_jobs_args_map = {}
    for jt in config.spec.jobs:
        if jt.name.startswith('preset.'):
            continue

        job_name = jt.name.replace('.', '_')
        job_full_path = context.dag_path + job_name
        args_map = job_conf_args2map(job_full_path)
        all_jobs_args_map[jt.name] = args_map

        phjobs_tmp = ['"{}": {{'.format(job_name)]
        for scope, args in args_map.items():
            phjobs_tmp.append('    "{}": {{'.format(scope))
            for k, v in args.items():
                phjobs_tmp.append('        "{}": "{}",'.format(k, v))
            phjobs_tmp.append('    },')
        phjobs_tmp.append('},')

        phjobs_lst += ['    '+s for s in phjobs_tmp]
    phjobs_str = 'phjobs = {\n' + '\n'.join(['    '+s for s in phjobs_lst]) + '\n    }'

    job_phjob_abs_path = os.path.abspath(__file__).split('/')
    job_phjob_abs_path[-1] = "phjob.py"
    job_phjob_abs_path = '/'.join(job_phjob_abs_path)
    with open(context.dag_path + DIR_NAME + "/phjob.py", 'w') as target:
        with open(job_phjob_abs_path) as source:
            data = source.read()
            data = data.replace("phjobs = {}", phjobs_str)
            target.write(data)

    # 3. /phmain.py file
    all_jobs_merge_args = {}
    for jt in config.spec.jobs:
        job_args = all_jobs_args_map.get(jt.name, {})
        result = dict([(k, v) for scope, args in job_args.items() for k, v in args.items()])
        merge_job_args(all_jobs_merge_args, result)

    must_args = [arg.strip() for arg in dv.PRESET_MUST_ARGS.split(',')]
    all_jobs_merge_args = dict([(k, v) for k, v in all_jobs_merge_args.items() if k not in must_args])

    click_option_lst = must_args + [k for k in all_jobs_merge_args]
    click_option_lst = ["@click.option('--{}')".format(k) for k in click_option_lst]
    click_option_str = '\n'.join(click_option_lst)

    job_phmain_abs_path = os.path.abspath(__file__).split('/')
    job_phmain_abs_path[-1] = "phmain.py"
    job_phmain_abs_path = '/'.join(job_phmain_abs_path)
    with open(context.dag_path + DIR_NAME + "/phmain.py", 'w') as target:
        with open(job_phmain_abs_path) as source:
            data = source.read()
            data = data.replace("@click.option('--job_id')", click_option_str)
            target.write(data)

    # 4. /args.properties file
    with open(context.dag_path + DIR_NAME + "/args.properties", "w") as f:
        for k, v in all_jobs_merge_args.items():
            f.write('--{}\n'.format(k))
            f.write('{}\n'.format(v))

    # 5. copy /phconf.yaml to phdags/
    with open(context.dag_path + DIR_NAME + "/phconf.yaml", 'w') as target:
            target.write(phconf_buf(context))



# -*- coding: utf-8 -*-

CLI_CLIENT_VERSION = '3.0.18'

CLI_BUCKET = 'ph-platform'
CLI_VERSION = "2020-11-11"

ASSUME_ROLE_ARN = 'YXJuOmF3cy1jbjppYW06OjQ0NDYwMzgwMzkwNDpyb2xlL1BoLUNsaQ=='
ASSUME_ROLE_EXTERNAL_ID = 'Ph-Cli'



# -*- coding: utf-8 -*-

from abc import abstractmethod


class AWSOperator(object):
    """
    封装 AWS 的常规操作和抽象方法
    """

    @abstractmethod
    def package(self, data):
        """
        对 lambda 的 layer 或者 code 打成 zip 包
        """
        pass

    @abstractmethod
    def create(self, data):
        """
        创建 lambda 的 layer 或者 code 或者 role
        """
        pass

    @abstractmethod
    def lists(self, data):
        """
        获取所有资源实例
        """
        pass

    @abstractmethod
    def get(self, data):
        """
        获取指定资源实例
        """
        pass

    @abstractmethod
    def update(self, data):
        """
        更新 lambda 的 layer 或者 code 或者 role
        """
        pass

    @abstractmethod
    def apply(self, data):
        """
        创建或更新 lambda 的 layer 或者 code 或者 role
        """
        pass

    @abstractmethod
    def stop(self, data):
        """
        使 lambda 或者 API Gateway 停止接受请求
        """
        pass

    @abstractmethod
    def start(self, data):
        """
        重新使 lambda 或者 API Gateway 接受请求
        """
        pass

    @abstractmethod
    def delete(self, data):
        """
        删除 lambda 的 layer, 或者 code, 或者 role, 或者 API Gateway
        """
        pass



import pytest
from phcli.ph_lmd.model.ph_gateway import PhGateway


@pytest.mark.dependency(depends=['test_ph_lambda.test_ph_lambda_create_s3'])
def test_ph_gateway_create():
    args = {
        "name": "test_ph_gateway_create",
        "rest_api_id": "2t69b7x032",
        "api_template": "s3://ph-platform/2020-08-10/template/python/phcli/lmd/jsonapi-openapi-template.yaml",
        "lambda_name": "test_ph_lambda_create",
        "alias_version": "current",
    }
    assert PhGateway().create(args) != {}


def test_ph_gateway_lists():
    assert PhGateway().lists({}) != {}


def test_ph_gateway_get():
    args = {
        "name": "test_ph_gateway_create",
        "rest_api_id": "2t69b7x032",
    }
    assert PhGateway().get(args) != {}


def test_ph_gateway_update():
    args = {
        "name": "test_ph_gateway_create",
        "rest_api_id": "2t69b7x032",
        "api_template": "s3://ph-platform/2020-08-10/template/python/phcli/lmd/jsonapi-openapi-template.yaml",
        "lambda_name": "test_ph_lambda_create",
        "alias_version": "current",
    }
    assert PhGateway().update(args) != {}


def test_ph_gateway_apply():
    args = {
        "name": "test_ph_gateway_create",
        "rest_api_id": "2t69b7x032",
        "api_template": "s3://ph-platform/2020-08-10/template/python/phcli/lmd/jsonapi-openapi-template.yaml",
        "lambda_name": "test_ph_lambda_create",
        "alias_version": "current",
    }
    assert PhGateway().apply(args) != {}


def test_ph_gateway_delete():
    args = {
        "name": "test_ph_gateway_create",
        "rest_api_id": "2t69b7x032",
    }
    assert PhGateway().delete(args) != {}




from phcli.ph_aws.ph_sts import PhSts
from phcli.ph_aws.ph_s3 import PhS3

phsts = PhSts()
phsts = phsts.assume_role('arn:aws-cn:iam::444603803904:role/Ph-Cli-Lmd', 'Ph-Cli-Lmd')

phs3 = PhS3()

TEST_BUCKET = 'ph-platform'
TEST_OBJECT = '2020-08-10/template/python/phcli/lmd/ph-lambda-deploy-template.yaml'


def test_ph_s3_list_buckets():
    assert phs3.list_buckets()


def test_ph_s3_open_object_by_lines():
    assert phs3.open_object_by_lines(TEST_BUCKET, TEST_OBJECT)


def test_ph_s3_download():
    phs3.download(TEST_BUCKET, TEST_OBJECT, 'test.yaml')


def test_ph_s3_create_presigned_url():
    url = phs3.s3_client.generate_presigned_url('get_object',
                                Params={'Bucket': 'ph-cmax-auto',
                                        'Key': 'test'},
                                ExpiresIn=3600)
    print(url)
test_ph_s3_create_presigned_url()




# -*- coding: utf-8 -*-
"""alfredyang@pharbers.com.

This module document the usage of class pharbers command context,
"""
import sys
import time
import click
from phcli.define_value import CLI_CLIENT_VERSION
from phcli.ph_max_auto.ph_hook.ph_hook import exec_before


def repl():
    try:
        lines = []
        line = input(">>> ").strip()
        while True:
            lines.append(line)
            if line.endswith(';') or line == '':
                break
            line = input("... ").strip()
        return ' '.join(lines)
    except Exception:
        pass


spark_session = None
def get_spark_session():
    global spark_session
    if not spark_session:
        spark_session = exec_before(
            name='phcli_sql_repl',
            job_id=time.time())['spark']()
    return spark_session


def exec_sql(sql):
    try:
        spark = get_spark_session()
        spark.sql(sql.strip(';')).show()
    except Exception as e:
        print(e)


@click.command("hive", short_help='hive sql 工具')
def main():
    print("Welcome to PhCli Hive SQL REPL v{}".format(CLI_CLIENT_VERSION))
    print("Can use standard SQL query pharbers hive.\n")

    try:
        while True:
            sql = repl()
            if sql:
                exec_sql(sql)

            input("\nPress `Enter` key to continue，Press `Ctrl+D` to exit").strip()
    except EOFError as _:
        sys.exit()


if __name__ == '__main__':
    main()



import click
from phcli.ph_dag.ph_airflow.__main__ import airflow


@click.group("dag", short_help='管理 DAG 相关技术栈')
def main():
    """
    管理 DAG 相关技术栈，如 Airflow
    """
    pass


main.add_command(airflow)

if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-


import base64
from phcli.ph_aws.ph_s3 import PhS3
from phcli.ph_aws.ph_sts import PhSts
from phcli.ph_storage import static as st


class PhS3Storage:

    __phsts = PhSts().assume_role(
        base64.b64decode(st.ASSUME_ROLE_ARN).decode(),
        st.ASSUME_ROLE_EXTERNAL_ID,
    )
    __phs3 = PhS3(phsts=__phsts)

    def upload(self, file, bucket_name, object_name):
        self.__phs3.upload(file, bucket_name, object_name)



# -*- coding: utf-8 -*-
"""alfredyang@pharbers.com.

This module document the usage of class pharbers command context,
"""
import base64

from phcli.ph_aws.ph_s3 import PhS3
from phcli.ph_aws.ph_sts import PhSts
from phcli.ph_max_auto import define_value as dv
from phcli.ph_max_auto.phcontext.ph_ide.ph_ide_c9 import PhIDEC9
from phcli.ph_max_auto.phcontext.ph_ide.ph_ide_jupyter import PhIDEJupyter
from phcli.ph_logs.ph_logs import phs3logger, LOG_DEBUG_LEVEL, LOG_INFO_LEVEL, LOG_WARN_LEVEL, LOG_ERROR_LEVEL


class PhContextFacade(object):
    def __init__(self, **kwargs):
        log_level = kwargs.pop('log_level')
        if log_level == 'info':
            log_level = LOG_INFO_LEVEL
        elif log_level == 'warn':
            log_level = LOG_WARN_LEVEL
        elif log_level == 'error':
            log_level = LOG_ERROR_LEVEL
        else:
            log_level = LOG_DEBUG_LEVEL

        self.logger = phs3logger(level=kwargs.get("log_level", log_level))
        self.phsts = PhSts().assume_role(
            base64.b64decode(dv.ASSUME_ROLE_ARN).decode(),
            dv.ASSUME_ROLE_EXTERNAL_ID,
        )
        self.phs3 = PhS3(phsts=self.phsts)
        self.__dict__.update(kwargs)
        self.ide_table = {
            'c9': PhIDEC9,
            'jupyter': PhIDEJupyter,
        }
        self.ide_inst = self.ide_table[self.ide](**self.__dict__)

    def command_create_exec(self):
        self.logger.debug("sub command create")
        self.ide_inst.create()

    def command_complete_exec(self):
        self.logger.debug("sub command complete")
        self.ide_inst.complete()

    def command_run_exec(self):
        self.logger.debug("sub command run")
        self.ide_inst.run()

    def command_combine_exec(self):
        self.logger.debug("sub command combine")
        self.ide_inst.combine()

    def command_dag_exec(self):
        self.logger.debug("sub command dag")
        self.ide_inst.dag()

    def command_publish_exec(self):
        self.logger.debug("sub command publish")
        self.ide_inst.publish()

    def command_recall_exec(self):
        self.logger.debug("sub command recall")
        self.ide_inst.recall()

    def command_online_run_exec(self):
        self.logger.debug("sub command online_run")
        self.ide_inst.online_run()

    def command_status_exec(self):
        self.logger.debug("sub command status")
        self.ide_inst.status()



# -*- coding: utf-8 -*-
"""alfredyang@pharbers.com.

This is job template for Pharbers Max Job
"""
import click
import traceback
from phjob import execute
from phcli.ph_logs.ph_logs import phs3logger
from phcli.ph_max_auto.ph_hook.ph_hook import exec_before, exec_after


@click.command()
@click.option('--job_id')
def debug_execute(**kwargs):
    try:
        args = {'name': 'write_asset'}

        args.update(kwargs)
        result = exec_before(**args)

        args.update(result if isinstance(result, dict) else {})
        result = execute(**args)

        args.update(result if isinstance(result, dict) else {})
        result = exec_after(outputs=[], **args)

        return result
    except Exception as e:
        logger = phs3logger(kwargs["job_id"])
        logger.error(traceback.format_exc())
        raise e


if __name__ == '__main__':
    debug_execute()





import uuid
from datetime import datetime
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy import Column, String, DateTime, Float


Base = declarative_base()


class DataSet(Base):
    __tablename__ = 'dataSet'

    id = Column(String, primary_key=True, default=lambda: str(uuid.uuid4()))
    parent = Column(String, default='{}')
    child = Column(String, default='{}')
    blockDs = Column(String)
    job = Column(String)
    sampleData = Column(String, default='{}')
    name = Column(String)
    schema = Column(String, default='{}')
    source = Column(String)
    storeType = Column(String, default='parquet')
    size = Column(Float)
    created = Column(DateTime, default=datetime.now())
    modified = Column(DateTime, default=datetime.now())
    description = Column(String)
    url = Column(String)
    tabName = Column(String)
    status = Column(String)
    mart = Column(String)
    assetDs = Column(String)
    colNames = Column(String, default='{}')
    length = Column(Float)

    def __repr__(self):
        return str(self.__dict__)


if __name__ == '__main__':
    ds = DataSet(id="id", parent="{}", child="{}", blockDs="blockDs", job="job", sampleData="{}",
                 name="name", schema="{}", source="source", storeType="storeType", size=0,
                 created=datetime.now(), modified=datetime.now(), description="description", url="url",
                 tabName="tabName", status="status", mart="mart", assetDs="assetDs", colNames="{}", length=0)
    print(ds)

    import base64
    from phcli.ph_db.ph_pg import PhPg

    pg = PhPg(
        base64.b64decode('cGgtZGItbGFtYmRhLmNuZ2sxamV1cm1udi5yZHMuY24tbm9ydGh3ZXN0LTEuYW1hem9uYXdzLmNvbS5jbgo=').decode('utf8')[:-1],
        base64.b64decode('NTQzMgo=').decode('utf8')[:-1],
        base64.b64decode('cGhhcmJlcnMK').decode('utf8')[:-1],
        base64.b64decode('QWJjZGUxOTYxMjUK').decode('utf8')[:-1],
        db=base64.b64decode('cGhlbnRyeQo=').decode('utf8')[:-1],
    )
    pg.insert(ds)
    pg.commit()



import base64
from sqlalchemy import create_engine
from sqlalchemy.orm import sessionmaker
from phcli.ph_db.ph_db_api import PhDBAPI


class PhPg(PhDBAPI):
    def __init__(self, host, port, user, passwd, db):
        self.engine = None
        self.session = None

        self.host = host
        self.port = port
        self.user = user
        self.passwd = passwd
        self.db = db

        self.engine = create_engine('postgresql://{user}:{passwd}@{host}:{port}/{db}'.format(**self.__dict__))
        Session = sessionmaker(bind=self.engine)
        self.session = Session()


if __name__ == '__main__':
    from phcli.ph_max_auto.ph_models.data_set import DataSet

    pg = PhPg(
        base64.b64decode('cGgtZGItbGFtYmRhLmNuZ2sxamV1cm1udi5yZHMuY24tbm9ydGh3ZXN0LTEuYW1hem9uYXdzLmNvbS5jbgo=').decode('utf8')[:-1],
        base64.b64decode('NTQzMgo=').decode('utf8')[:-1],
        base64.b64decode('cGhhcmJlcnMK').decode('utf8')[:-1],
        base64.b64decode('QWJjZGUxOTYxMjUK').decode('utf8')[:-1],
        db=base64.b64decode('cGhlbnRyeQo=').decode('utf8')[:-1],
    )

    print(pg.tables())

    # print(pg.insert(DataSet(job='job')))

    query = pg.query(DataSet(job="job"))
    for q in query:
        print(q)

    pg.commit()




# -*- coding: utf-8 -*-

import subprocess
from phcli.ph_storage import static as st


class PhHdfsStorage:
    """
    HDFS 存储，操作，目前只有Buck Up数据到S3
    """

    def __init__(self, local_storage=None, s3_storage=None):
        self.__local_storage = local_storage
        self.__s3_storage = s3_storage

    def get_path(self, path):
        """
        根据传入路径，列出该目录下的所有
        return [string]
        """
        cmd = "hdfs dfs -ls -R " + path + " | awk '{if($1!~/^d/&&!/inprogress/)print $8}'"
        res = subprocess.Popen(cmd, shell=True,
                               stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
        paths = []
        for line in res.stdout.readlines():
            content = line.decode("utf-8").replace("\n", "")
            if content != "" and content.find("WARN") == -1:
                paths.append(content)
        return paths

    def __down_load(self, path, output):
        cmd = "hdfs dfs -get " + path + " " + output + ""
        res = subprocess.Popen(cmd, shell=True,
                               stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
        for line in res.stdout.readlines():
            print(line)

        return output

    def back_up(self, paths):
        if self.__local_storage is None or self.__s3_storage is None:
            return False
        try:
            hdfs_path = []
            for item in paths:
                if item[0] == "u":
                    hdfs_path.extend(self.get_path(item[1:]))
                else:
                    hdfs_path.extend(self.get_path(item))
            for item in hdfs_path:
                file_name = item.split("/")[-1]
                sub_path = "/".join(
                    [elem for elem in item.split("/")[1:-1] if elem.find(":") < 0])
                upload_path = (st.UPLOADPATH + "/" + sub_path + "/" + file_name).replace("//", "/")
                download_path = st.createDownLoadPath(self.__local_storage, sub_path)
                self.__down_load(item, download_path)
                self.__s3_storage.upload(download_path + "/" + file_name, st.BUCKET, upload_path)
                self.__local_storage.remove(download_path + "/" + file_name)
        except BaseException as e:
            print("back_up Error: ", e)
            return False
        else:
            return True

    def clean_hdfs(self, paths):
        try:
            print("clean hdfs start")
            for path in paths:
                if path[0] == "u":
                    cmd = "hdfs dfs -ls -R " + path[1:] + " | awk '{if($1!~/^d/&&!/inprogress/)print $8}' | xargs hdfs dfs -rm -R -skipTrash"
                    print("cmd ==>" + cmd)
                    res = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
                    for line in res.stdout.readlines():
                        print(line.decode("utf-8").replace("\n", ""))
                else:
                    cmd = "hdfs dfs -ls -R " + path + " | awk '{if($1!~/^d/&&!/inprogress/)print $8}' | xargs hdfs dfs -rm -R -skipTrash"
                    print("cmd ==>" + cmd)
                    res = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
                    for line in res.stdout.readlines():
                        print(line.decode("utf-8").replace("\n", ""))
            print("clean hdfs end")
        except BaseException as e:
            print("clean_hdfs Error: ", e)
            return False
        else:
            return True



# -*- coding: utf-8 -*-

import click
from phcli.ph_admin.ph_partner import main as partner_main
from phcli.ph_admin.ph_user import main as user_main
from phcli.ph_admin.ph_role import main as role_main
from phcli.ph_admin.ph_user import main as scope_main


@click.group("admin", short_help='管理员管理工具')
def main():
    """
    本脚本用于管理用户系统，如公司，用户，角色等
    """
    pass


main.add_command(partner_main)
main.add_command(user_main)
main.add_command(scope_main)
main.add_command(role_main)


if __name__ == '__main__':
    main()



import boto3

def get_args_from_ssm(kwargs):
    ssm_client = boto3.client('ssm', region_name='cn-northwest-1',
                                          aws_access_key_id="AKIAWPBDTVEANKEW2XNC",
                                          aws_secret_access_key="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")
    response = ssm_client.get_parameter(
        Name=kwargs.get('job_args_name')
    )
    args = eval(response['Parameter']['Value'])
    return args

def delete_args_from_ssm(kwargs):
    ssm_client = boto3.client('ssm', region_name='cn-northwest-1',
                              aws_access_key_id="AKIAWPBDTVEANKEW2XNC",
                              aws_secret_access_key="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")
    ssm_client.delete_parameter(
        Name=kwargs.get('job_args_name')
    )

if __name__ == '__main__':
    asd = get_args_from_ssm({1:1})
    print(asd)


# -*- coding: utf-8 -*-

from phcli.ph_lmd.runtime.lambda_rt import LambdaRuntime


class PythonRT(LambdaRuntime):
    __runtime_name = "python"

    def pkg_layer(self, data):
        """
        组织 layer 的打包逻辑
        :param data
            :arg is_pipenv: 是否使用的 pipenv 构建的项目，默认为 False
            :arg lib_path: 仅当 is_pipenv = False 时有效，指定 python 的 lib 位置
            :arg package_name: 打包的名称
        """

        self._package_cmds = [
            "mkdir -p %s/%s" % (self._package_root, self.__runtime_name),
        ]

        if data.get("is_pipenv", False):
            self._package_cmds.extend([
                "pipenv lock -r | grep '==' | awk -F '==' '{print $1}' |" +
                "xargs -I '{}' cp -r ${VIRTUAL_ENV}/lib/python3.8/site-packages/{} %s/%s/" % (
                    self._package_root, self.__runtime_name),
            ])
        else:
            self._package_cmds.extend([
                "cp -r %s/* %s/%s/" % (data["lib_path"], self._package_root, self.__runtime_name),
            ])

        self._package_cmds.extend([
            "cd %s && zip -r -q ../package.zip . && cd -" % self._package_root,
            "rm -rf %s" % self._package_root,
        ])

        self.package(data["package_name"])

    def pkg_code(self, data):
        """
        组织 code 的打包逻辑
        :param data
            :arg code_path: 需要打包的代码位置
            :arg package_name: 打包的名称
        """

        self._package_cmds = [
            "zip -r -q package.zip %s" % data["code_path"],
        ]

        self.package(data["package_name"])



# -*- coding: utf-8 -*-

import xlrd
import yaml

test_file = r'../../../file/ph_data_clean/s3_test_data/product-product-test.yaml'
# test_file = r'../../file/ph_data_clean/s3_test_data/universe-universe-test.yaml'
readbook = xlrd.open_workbook(r'../../../file/ph_data_clean/s3_primitive_data/Product_origin_file.xlsx')
# readbook = xlrd.open_workbook(r'..\..\file\ph_data_clean\s3_primitive_data\Universe_origin_file.xlsx')
sheet = readbook.sheet_by_index(0)
nrows = sheet.nrows
ncols = sheet.ncols
print(ncols)
test_data_key = sheet.row_values(0)


def get_universe_test_data():

    test_data_lst = []

    for i in range(50, 150):
        value = sheet.row_values(i)
        test_data = {'data': {},
                     'metadata': {'fileName': 'Book1',
                                  # 'providers': ['universe', 'universe'],
                                  'providers': ['product', 'product'],
                                  'sheetName': 'sheet1'}}
        for j in range(0, ncols-1):
            real_key = str(j) + '#' + str(test_data_key[j])
            test_data['data'][real_key] = value[j]

        # 加上tag
        test_data['data']['_Tag'] = 'yes'

        test_data_str = str(test_data['data'])
        test_data['data'] = test_data_str

        meta_data_str = str(test_data['metadata'])
        test_data['metadata'] = meta_data_str

        test_data_lst.append(test_data)
    return test_data_lst


if __name__ == '__main__':
    with open(test_file, 'a', encoding='UTF-8') as file:
        yaml.dump(get_universe_test_data(), file, default_flow_style=False, encoding='utf-8', allow_unicode=True)



# -*- coding: utf-8 -*-

import click
from phcli.ph_logs.ph_logs import *
from phcli.ph_aws.ph_s3 import PhS3


@click.command("logs", short_help='查看生产环境的运行日志')
@click.option('--follow/--no-follow', '-f', default=False,
              help='流式打印日志(暂不支持)')
@click.argument('job_id', nargs=1)
def main(follow, job_id):
    """
    本脚本用于查看生产环境的运行日志
    """
    phs3 = PhS3(access_key=PH_CLI_ACCESS_KEY, secret_key=PH_CLI_SECRET_KEY)
    print(phs3.open_object(CLI_BUCKET, LOG_PATH.format(CLI_VERSION, job_id + '.log')))


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-

from phcli.ph_data_clean.clean.common_func import *
from phcli.ph_data_clean.clean.data_clean import DataClean


class ProductDataClean(DataClean):
    """
    product 源数据的清洗规则
    """

    def product_clean_process(self, *args, **kwargs):
        raw_data = args[1]
        final_data = {}

        for raw_data_key in raw_data.keys():
            final_data_key = raw_data_key.split("#")[-1].replace('\n', '').strip()
            final_data[final_data_key] = raw_data[raw_data_key]

        if "_tag" in final_data:
            final_data["TAG"] = final_data.pop("_tag")

        return final_data

    def define_tag_err_for_prod(self, *args, **kwargs):
        mapping = args[0]
        raw_data = args[1]
        final_data = kwargs["prev"]

        # define tag and err msg
        tag_value = Tag.UNDEFINED
        if raw_data == {}:  # 若原始数据为空
            tag_value = Tag.EMPTY_DICT
            error_msg = 'Error message: empty raw_data'
        elif final_data == {}:  # 若最终字典没有内容
            tag_value = Tag.EMPTY_DICT
            error_msg = 'Error message: no mapping found'
        else:
            warning_col_lst = []
            for m in mapping:
                if m["not_null"] and (final_data[m["col_name"]] in [None, "", "/"]):
                    warning_col_lst.append(m["col_name"])

            # 如果是空字典（没有warning）
            if not warning_col_lst:
                tag_value = Tag.SUCCESS
                error_msg = 'Success'

            # 如果不是空字典（有warning）
            else:
                tag_value = Tag.WARNING
                error_msg = 'Warning: col-missing ' + str(warning_col_lst)

        return CleanResult(data=final_data,
                           metadata={},
                           raw_data=raw_data,
                           tag=tag_value,
                           err_msg=error_msg),

    process = [
        product_clean_process,
        define_tag_err_for_prod,
    ]



import os
import subprocess

from phcli.ph_errs.ph_err import *
from .ph_rt_base import PhRTBase
from phcli.define_value import CLI_CLIENT_VERSION
from phcli.ph_max_auto import define_value as dv
from phcli.ph_max_auto.ph_config.phconfig.phconfig import PhYAMLConfig


class PhRTR(PhRTBase):
    def __init__(self, **kwargs):
        self.__dict__.update(kwargs)

    def c9_create_phmain(self, path=None):
        if not path:
            path = self.job_path

        config = PhYAMLConfig(path)
        config.load_yaml()
        f_lines = self.phs3.open_object_by_lines(dv.TEMPLATE_BUCKET, dv.CLI_VERSION + dv.TEMPLATE_PHMAIN_FILE_R)
        with open(path + "/phmain.R", "w") as file:
            options_args = []
            for arg in config.spec.containers.args:
                options_args.append('c("key"="{key}", "desc"="参数{key}")'.format(key=arg.key))
            for output in config.spec.containers.outputs:
                options_args.append('c("key"="{key}", "desc"="参数{key}")'.format(key=output.key))

            for line in f_lines:
                line = line + "\n"
                if "$options_args" in line:
                    line = line.replace("$options_args", ',\n\t'.join(options_args))
                file.write(line)

    def jupyter_to_c9(self, dag_full_path, **kwargs):
        im = kwargs['im']
        om = kwargs['om']
        ipynb_dict = kwargs['ipynb_dict']

        self.phs3.download(dv.TEMPLATE_BUCKET, dv.CLI_VERSION + dv.TEMPLATE_PHJOB_FILE_R, dag_full_path + "/phjob.R")
        config = PhYAMLConfig(dag_full_path)
        config.load_yaml()

        with open(dag_full_path + "/phjob.R", "a") as file:
            file.write("execute <- function(")
            file.write(", ".join(im.keys()))
            file.write(", ")
            file.write(", ".join(om.keys()))
            file.write("){\n")

            # copy 逻辑代码
            for cell in ipynb_dict['cells'][2:]:
                for row in cell['source']:
                    file.write('    '+row)
                file.write('\r\n')
                file.write('\r\n')

            file.write('}')

    def jupyter_create(self, **kwargs):
        path = self.job_path + ".ipynb"
        dir_path = "/".join(path.split('/')[:-1])
        subprocess.call(['mkdir', '-p', dir_path])

        f_lines = self.phs3.open_object_by_lines(dv.TEMPLATE_BUCKET, dv.CLI_VERSION + dv.TEMPLATE_JUPYTER_R_FILE)
        with open(path, "w") as file:
            for line in f_lines:
                line = line.replace('$name', self.name) \
                            .replace('$runtime', self.runtime) \
                            .replace('$command', self.command) \
                            .replace('$timeout', str(self.timeout)) \
                            .replace('$user', os.getenv('USER', 'unknown')) \
                            .replace('$group', self.group) \
                            .replace('$ide', self.ide) \
                            .replace('$access_key', os.getenv('AWS_ACCESS_KEY_ID', "NULL")) \
                            .replace('$secret_key', os.getenv('AWS_SECRET_ACCESS_KEY', "NULL"))

                file.write(line)

    def create(self, **kwargs):
        if self.ide == 'jupyter':
            self.jupyter_create(**kwargs)
        else:
            raise exception_function_not_implement

    def submit_run(self, **kwargs):
        submit_conf = {}
        submit_file = {
            "files": self.submit_prefix + "phjob.R",
        }
        submit_main = self.submit_prefix + "phmain.R"

        super().submit_run(submit_conf=submit_conf,
                           submit_file=submit_file,
                           submit_main=submit_main)

    def script_run(self, **kwargs):
        self.phs3.download(dv.TEMPLATE_BUCKET, dv.CLI_VERSION + self.s3_job_path + "/phmain.R", 'phmain.R')
        self.phs3.download(dv.TEMPLATE_BUCKET, dv.CLI_VERSION + self.s3_job_path + "/phjob.R", 'phjob.R')
        entrypoint = ['Rscript', './phmain.R']
        super().script_run(entrypoint=entrypoint)



# -*- coding: utf-8 -*-


class PhException(Exception):
    def __init__(self, *args, **kwargs):
        self.msg = args[0] if args else 'Pharbers Unknow Exception'

    def __str__(self):
        return repr(self.msg)


class PhRuntimeError(RuntimeError):
    """The Pharbers Exceptions

        Args:
            code: the number of exceptions
            msg: the message of exceptions

    """

    def __init__(self, code, msg):
        self.code = code
        self.msg = msg


# Maxauto 1 - 99
exception_file_already_exist = PhRuntimeError(-1, "The File Is Already Exists")
exception_file_not_exist = PhRuntimeError(-2, "The File Is Not Exists")
exception_function_not_implement = PhRuntimeError(-3, "the function is not implement, please contact "
                                                      "alfredyang@pharbers.com")
exception_not_found_preset_job = PhRuntimeError(-4, "this preset job is non-existent")

# Lambda 100 - 199



import base64
from datetime import datetime
from phcli.ph_db.ph_pg import PhPg
from phcli.ph_max_auto.ph_models.data_set import DataSet


def lineage(job_id, kwargs):

    # job_id 为空判定为测试环境，不管理系统
    if not job_id:
        return

    outputs = kwargs.pop('outputs', [])
    inputs = list(set(kwargs.keys()).difference(outputs))
    outputs = [output for output in outputs if kwargs[output] and str(kwargs[output]).startswith('s3a://')]
    inputs = [input for input in inputs if kwargs[input] and str(kwargs[input]).startswith('s3a://')]

    # 没有输出需要记录，直接退出
    if not outputs:
        return

    pg = PhPg(
        base64.b64decode('cGgtZGItbGFtYmRhLmNuZ2sxamV1cm1udi5yZHMuY24tbm9ydGh3ZXN0LTEuYW1hem9uYXdzLmNvbS5jbgo=').decode(
            'utf8')[:-1],
        base64.b64decode('NTQzMgo=').decode('utf8')[:-1],
        base64.b64decode('cGhhcmJlcnMK').decode('utf8')[:-1],
        base64.b64decode('QWJjZGUxOTYxMjUK').decode('utf8')[:-1],
        db=base64.b64decode('cGhlbnRyeQo=').decode('utf8')[:-1],
    )

    input_ids = []
    for input in inputs:
        obj = pg.query(DataSet(), source=kwargs[input])
        if obj:
            obj_id = obj[0].id
        else:
            obj = pg.insert(DataSet(job=job_id, name=input, source=kwargs[input]))
            obj_id = obj.id
        input_ids.append(obj_id)

    for output in outputs:
        obj = pg.query(DataSet(), source=kwargs[output])
        if obj:
            obj = obj[0]
            obj.parent = input_ids
            obj.modified = datetime.now()
            pg.update(obj)
        else:
            pg.insert(DataSet(parent=input_ids, job=job_id, name=output, source=kwargs[output]))

    pg.commit()





# -*- coding: utf-8 -*-

from phcli.ph_errs.ph_err import PhException
from phcli.ph_lmd.runtime import python_rt
from phcli.ph_lmd.runtime import nodejs_rt
from phcli.ph_lmd.runtime import go_rt


def get_short_rt(runtime):
    if "python" in runtime:
        return "python"
    elif "nodejs" in runtime:
        return "nodejs"
    elif "go" in runtime:
        return "go"
    raise PhException("Invalid runtime")


def get_rt_inst(runtime):
    rt_table = {
        'python': python_rt.PythonRT,
        'nodejs': nodejs_rt.NodejsRT,
        'go': go_rt.GoRT,
    }

    rt = rt_table[get_short_rt(runtime)]
    return rt()



# -*- coding: utf-8 -*-

from phcli.ph_data_clean.clean.common_func import *
from phcli.ph_data_clean.clean.data_clean import DataClean


class CpaGycDataClean(DataClean):
    """
    CPA & GYC 等源数据的清洗规则
    """

    process = [
        check_format,
        change_key,
        change_year_month,
        change_sales_tag,
        reformat_int,
        define_tag_err,
    ]



import base64
from phcli.ph_db.ph_pg import PhPg


def conn_pg():
    return PhPg(
        base64.b64decode('cGgtZGItbGFtYmRhLmNuZ2sxamV1cm1udi5yZHMuY24tbm9ydGh3ZXN0LTEuYW1hem9uYXdzLmNvbS5jbgo=').decode('utf8')[:-1],
        base64.b64decode('NTQzMgo=').decode('utf8')[:-1],
        base64.b64decode('cGhhcmJlcnMK').decode('utf8')[:-1],
        base64.b64decode('QWJjZGUxOTYxMjUK').decode('utf8')[:-1],
        db=base64.b64decode('cGhjb21tb24K').decode('utf8')[:-1],
    )


pg = conn_pg()



import subprocess

class PhInitConf(object):

    def init_conf(cluster_id):
        cmd1 = "aws s3 cp s3://ph-platform/2020-11-11/emr/remoteConfig/"+ cluster_id +"/etc/hadoop/conf/ hadoop/conf/ --recursive"
        cmd2 = "sudo cp hadoop/conf/* /etc/hadoop/conf/"
        cmd3 = "aws s3 cp s3://ph-platform/2020-11-11/emr/remoteConfig/"+ cluster_id +"/etc/spark/conf/ spark/conf/ --recursive"
        cmd4 = "sudo cp spark/conf/* /etc/spark/conf/"
        cmd = cmd1 + " && " + cmd2 + " && " + cmd3 + " && " + cmd4
        subprocess.call(cmd, shell=True)



# -*- coding: utf-8 -*-
"""alfredyang@pharbers.com.

This module document the YAML Job Args for Pharbers jobs
"""


class PhYAMLJobArgs(object):
    def __init__(self, dt):
        self.key = ""
        self.value = ""
        self.dict2obj(dt)

    def dict2obj(self, dt):
        self.__dict__.update(dt)

    def __str__(self):
        return str(self.__dict__)



# -*- coding: utf-8 -*-
import boto3
import botocore.exceptions
from phcli.ph_aws.aws_root import PhAWS


class PhSts(PhAWS):
    def __init__(self, **kwargs):
        self.credentials = None
        self.access_key = kwargs.get('access_key', None)
        self.secret_key = kwargs.get('secret_key', None)

    def get_cred(self):
        if self.credentials:
            return {
                'region_name': 'cn-northwest-1',
                'aws_access_key_id': self.credentials['AccessKeyId'],
                'aws_secret_access_key': self.credentials['SecretAccessKey'],
                'aws_session_token': self.credentials['SessionToken'],
            }

    def assume_role(self, role_arn, external_id):
        if self.access_key and self.secret_key:
            sts_client = boto3.client('sts', region_name='cn-northwest-1',
                                      aws_access_key_id=self.access_key,
                                      aws_secret_access_key=self.secret_key)
        else:
            sts_client = boto3.client('sts', region_name='cn-northwest-1')

        try:
            assumed_role_object = sts_client.assume_role(
                RoleArn=role_arn,
                RoleSessionName=external_id,
                ExternalId=external_id,
            )
        except botocore.exceptions.ClientError:
            self.credentials = {}
        else:
            self.credentials = assumed_role_object['Credentials']

        return self



import os
import sys
import ast
import subprocess
import boto3
import json
import time
import uuid
import base64
from enum import Enum

from phcli.ph_errs.ph_err import *
from phcli.ph_max_auto import define_value as dv
from phcli import define_value as phcli_dv
from phcli.ph_max_auto.ph_config.phconfig.phconfig import PhYAMLConfig
from phcli.ph_max_auto.ph_preset_jobs.preset_job_factory import preset_factory
from phcli.ph_aws.ph_sts import PhSts
from phcli.ph_logs.ph_logs import phs3logger, LOG_DEBUG_LEVEL

logger = phs3logger("hbzhao12345", LOG_DEBUG_LEVEL)
class PhCompleteStrategy(Enum):
    S2C = 'special to common'
    C2S = 'common to special'
    KEEP = 'keep still'


class PhIDEBase(object):
    job_prefix = "/phjobs/"
    combine_prefix = "/phcombines/"
    dag_prefix = "/phdags/"
    upload_prefix = "/upload/"

    def __init__(self, **kwargs):
        self.__dict__.update(kwargs)
        self.__dict__.update(self.get_absolute_path())
        self.logger.debug('maxauto PhIDEBase init')
        self.logger.debug(self.__dict__)

    def get_workspace_dir(self):
        return os.getenv(dv.ENV_WORKSPACE_KEY, dv.ENV_WORKSPACE_DEFAULT)

    def get_current_project_dir(self):
        return os.getenv(dv.ENV_CUR_PROJ_KEY, dv.ENV_CUR_PROJ_DEFAULT)

    def get_absolute_path(self):
        if 'name' not in self.__dict__ and 'job_full_name' in self.__dict__:
            self.name = self.job_full_name

        project_path = self.get_workspace_dir() + '/' + self.get_current_project_dir()
        job_path = project_path + self.job_prefix + (self.group + '/' if 'group' in self.__dict__.keys() else '') + self.name
        combine_path = project_path + self.combine_prefix + self.name + '/'
        dag_path = project_path + self.dag_prefix + self.name + '/'
        upload_path = project_path + self.upload_prefix + self.name + '/'

        return {
            'project_path': project_path,
            'job_path': job_path,
            'combine_path': combine_path,
            'dag_path': dag_path,
            'upload_path': upload_path,
        }

    def check_path(self, path):
        suffixs = ['', '.ipynb']
        for suffix in suffixs:
            tmp_path = path + suffix
            if os.path.exists(tmp_path):
                override = input('Job "' + tmp_path + '" already existed，want to override？(Y/*)')
                if override.upper() == 'Y':
                    subprocess.call(["rm", "-rf", tmp_path])
                else:
                    self.logger.error('Termination Create')
                    sys.exit()

    def table_driver_runtime_inst(self, runtime):
        from ..ph_runtime.ph_rt_python3 import PhRTPython3
        from ..ph_runtime.ph_rt_r import PhRTR
        table = {
            "python3": PhRTPython3,
            "r": PhRTR,
        }
        return table[runtime]

    def create(self, **kwargs):
        """
        默认的创建过程
        """
        self.logger.debug('maxauto 默认的 create 实现')
        self.logger.debug(self.__dict__)

        runtime_inst = self.table_driver_runtime_inst(self.runtime)
        runtime_inst(**self.__dict__).create()

    def choice_complete_strategy(self, special_path, common_path):
        special_last_modify_time = os.path.getmtime(special_path) if os.path.exists(special_path) else 0
        common_last_modify_time = os.path.getmtime(common_path) if os.path.exists(common_path) else 0

        if special_last_modify_time > common_last_modify_time:
            return PhCompleteStrategy.S2C
        elif special_last_modify_time < common_last_modify_time:
            return PhCompleteStrategy.C2S
        else:
            return PhCompleteStrategy.KEEP

    def complete(self, **kwargs):
        """
        默认的补全过程
        """
        self.logger.debug('maxauto 默认的 complete 实现')
        self.logger.debug(self.__dict__)

    def run(self, **kwargs):
        """
        默认的运行过程
        """
        self.logger.debug('maxauto 默认的 run 实现')
        self.logger.debug(self.__dict__)

        config = PhYAMLConfig(self.job_path)
        config.load_yaml()

        if config.spec.containers.repository == "local":
            timeout = float(config.spec.containers.timeout) * 60
            entry_runtime = config.spec.containers.runtime
            entry_runtime = self.table_driver_runtime_inst(self.runtime)().table_driver_runtime_binary(entry_runtime)
            entry_point = config.spec.containers.code
            entry_point = self.job_path + '/' + entry_point

            cb = [entry_runtime, entry_point]
            for arg in config.spec.containers.args:
                cb.append("--" + arg.key)
                cb.append(str(arg.value))
            for output in config.spec.containers.outputs:
                cb.append("--" + output.key)
                cb.append(str(output.value))
            prc = subprocess.run(cb, timeout=timeout, stderr=subprocess.PIPE)
            if prc.returncode != 0:
                raise Exception(prc.stderr.decode('utf-8'))
            return
        else:
            raise exception_function_not_implement

    def combine(self, **kwargs):
        """
        默认的关联过程
        """
        self.logger.debug('maxauto 默认的 combine 实现')
        self.logger.debug(self.__dict__)

        self.check_path(self.combine_path)
        subprocess.call(["mkdir", "-p", self.combine_path])

        def extract_jobs(jobs_str):
            jobs_lst = [job.strip() for job in jobs_str.split(',')]
            jobs = '\n    '.join(['- name: ' + job for job in jobs_lst])
            linkage = ' >> '.join(jobs_lst)
            linkage = '"' + linkage + '"'
            return jobs, linkage

        jobs_str, linkage_str = extract_jobs(self.jobs)
        f_lines = self.phs3.open_object_by_lines(dv.TEMPLATE_BUCKET, dv.CLI_VERSION + dv.TEMPLATE_PHDAG_FILE)
        with open(self.combine_path + "/phdag.yaml", "w") as file:
            for line in f_lines:
                line = line.replace("$name", self.name) \
                            .replace("$dag_owner", self.owner) \
                            .replace("$dag_tag", self.tag) \
                            .replace("$dag_timeout", self.timeout) \
                            .replace("$linkage", linkage_str) \
                            .replace("$jobs", jobs_str)
                file.write(line + "\n")

    def get_dag_py_file_name(self, key):
        return "ph_dag_" + key + ".py"

    def dag(self, **kwargs):
        """
        默认的DAG过程
        """
        self.logger.debug('maxauto 默认的 dag 实现')
        self.logger.debug(self.__dict__)

        self.check_path(self.dag_path)
        subprocess.call(["mkdir", "-p", self.dag_path])

        config = PhYAMLConfig(self.combine_path, "/phdag.yaml")
        config.load_yaml()

        def get_jobs_conf(config):
            def get_job_conf(name):
                job_full_path = self.project_path + self.job_prefix + name.replace('.', '/')
                if name.startswith('preset.'):
                    job_name = name.lstrip('preset.')
                    ipt_module = __import__('phcli.ph_max_auto.ph_preset_jobs.%s' % (job_name.lower()))
                    ipt_module = getattr(ipt_module, 'ph_max_auto')
                    ipt_module = getattr(ipt_module, 'ph_preset_jobs')
                    ipt_module = getattr(ipt_module, job_name)
                    phconf_buf = getattr(ipt_module, 'phconf_buf')

                    config = PhYAMLConfig()
                    config.load_yaml(phconf_buf(self))
                    return {
                        'name': config.metadata.name,
                        'ide': 'preset',
                        'runtime': config.spec.containers.runtime,
                        'command': config.spec.containers.command,
                        'timeout': config.spec.containers.timeout,
                    }
                elif os.path.isdir(job_full_path):
                    config = PhYAMLConfig(job_full_path)
                    config.load_yaml()
                    return {
                        'name': config.metadata.name,
                        'ide': 'c9',
                        'runtime': config.spec.containers.runtime,
                        'command': config.spec.containers.command,
                        'timeout': config.spec.containers.timeout,
                    }
                else:
                    raise Exception("{} job not found".format(name))

            result = {}
            for job in config.spec.jobs:
                result[job.name] = get_job_conf(job.name)
            return result

        def copy_jobs(jobs_conf):
            ide_dag_copy_job_func_table = {
                'c9': self.ide_table['c9'](**self.__dict__).dag_copy_job,
                'preset': preset_factory,
            }

            # 必须先 copy 所有非 preset 的 job
            for name, job_info in jobs_conf.items():
                if job_info['ide'] != 'preset':
                    func = ide_dag_copy_job_func_table[job_info['ide']]
                    func(job_name=name, **job_info)

            # 然后再 copy 所有 preset 的 job
            for name, job_info in jobs_conf.items():
                if job_info['ide'] == 'preset':
                    func = ide_dag_copy_job_func_table[job_info['ide']]
                    func(self, job_name=name, **job_info)

        def write_dag_pyfile(jobs_conf):
            timeout = config.spec.dag_timeout if config.spec.dag_timeout else sum([float(job['timeout']) for _, job in jobs_conf.items()])
            w = open(self.dag_path + self.get_dag_py_file_name(config.spec.dag_id), "a")
            f_lines = self.phs3.open_object_by_lines(dv.TEMPLATE_BUCKET, dv.CLI_VERSION + dv.TEMPLATE_PHGRAPHTEMP_FILE)
            for line in f_lines:
                line = line + "\n"
                w.write(
                    line.replace("$alfred_dag_owner", str(config.spec.owner)) \
                        .replace("$alfred_email_on_failure", str(config.spec.email_on_failure)) \
                        .replace("$alfred_email_on_retry", str(config.spec.email_on_retry)) \
                        .replace("$alfred_email", str(config.spec.email)) \
                        .replace("$alfred_retries", str(config.spec.retries)) \
                        .replace("$alfred_retry_delay", str(config.spec.retry_delay)) \
                        .replace("$alfred_dag_id", str(config.spec.dag_id)) \
                        .replace("$alfred_dag_tags", str(','.join(['"'+tag+'"' for tag in config.spec.dag_tag.split(',')]))) \
                        .replace("$alfred_schedule_interval", str(config.spec.schedule_interval)) \
                        .replace("$alfred_description", str(config.spec.description)) \
                        .replace("$alfred_dag_timeout", str(timeout)) \
                        .replace("$alfred_start_date", str(config.spec.start_date))
                )

            jf = self.phs3.open_object_by_lines(dv.TEMPLATE_BUCKET, dv.CLI_VERSION + dv.TEMPLATE_PHDAGJOB_FILE)
            for jt in config.spec.jobs:
                job_name = jt.name.replace('.', '_')

                for line in jf:
                    line = line + "\n"
                    w.write(
                        line.replace("$alfred_jobs_dir", str(self.name)) \
                            .replace("$alfred_name", str(job_name))
                    )

            for linkage in config.spec.linkage:
                w.write(linkage.replace('.', '_'))
                w.write("\n")

            w.close()

        jobs_conf = get_jobs_conf(config)
        copy_jobs(jobs_conf)
        write_dag_pyfile(jobs_conf)

    def publish(self, **kwargs):
        """
        默认的发布过程
        """
        self.logger.debug('maxauto 默认的 publish 实现')
        self.logger.debug(self.__dict__)

        def write_args(args_list, job_args):
            keys = []
            values = []
            # 获取list中的参数并 生成一个dict
            for arg in args_list:
                if args_list.index(arg) % 2 == 0:
                    keys.append(arg)
                elif args_list.index(arg) % 2 == 1:
                    values.append(arg)
            args = zip(keys, values)
            args_dict = dict(args)
            # 获取生成dict中的参数, 将传进来的dict进行替换
            for key in args_dict.keys():
                if key in job_args.keys():
                    args_dict[key] = job_args[key]
            args_dict_keys = list(args_dict)
            args_dict_values = list(args_dict.values())
            # 将替换好的dict转换成list
            final_args_list = []
            for i in range(len(args_dict_keys)):
                final_args_list.append(args_dict_keys[i])
                final_args_list.append(args_dict_values[i])
            return final_args_list

        def write_data(jobs, definition, s3_dag_path, dag_path, excution_name, job_args, random_num=""):
            for job in jobs:
                args_path = dag_path +job + "/" + "args.properties"
                Parameters = {
                    "ClusterId.$": "$.clusterId",
                    "Step": {
                        "Name": "My EMR step",
                        "ActionOnFailure": "CONTINUE",
                        "HadoopJarStep": {
                            "Jar": "command-runner.jar",
                            "Args": ["spark-submit",
                                     "--deploy-mode", "cluster",
                                     "--py-files",
                                     "s3://ph-platform/2020-11-11/jobs/python/phcli/common/phcli-"+ phcli_dv.CLI_CLIENT_VERSION +"-py3.8.egg," + s3_dag_path + job + "/phjob.py",
                                     s3_dag_path + job + "/phmain.py",
                                     "--owner", "default_owner",
                                     "--dag_name", s3_dag_path.split('/')[-2],
                                     "--run_id", s3_dag_path.split('/')[-2] + "_" +excution_name,
                                     "--job_full_name", job,
                                     "--job_id", "not_implementation",
                                     ]
                        }
                    }
                }
                if job in definition['States'].keys():
                    if jobs.index(job) + 1 == len(jobs):
                        definition['States'][job]['End'] = True
                        if not job.startswith('['):
                            definition['States'][job]['Type'] = "Task"
                            definition['States'][job]['Resource'] = Resource
                            args_list = []
                            with open(args_path, "r") as args_file:
                                arg_line = args_file.readline()
                                while arg_line:
                                    args_list.append(arg_line.rstrip('\n'))
                                    arg_line = args_file.readline()
                            # 替换args.properties中的参数
                            final_args_list = write_args(args_list, job_args)
                            Parameters['Step']['HadoopJarStep']['Args'][len(Parameters['Step']['HadoopJarStep']['Args'])
                                                                        :len(Parameters['Step']['HadoopJarStep']['Args'])] = final_args_list
                            definition['States'][job]['Parameters'] = Parameters
                            definition['States'][job]['ResultPath'] = "$.firstStep"
                            definition['States'][job]['Retry'] = [
                                                                    {
                                                                      "ErrorEquals": [ "States.ALL" ],
                                                                      "IntervalSeconds": 1,
                                                                      "MaxAttempts": 1,
                                                                      "BackoffRate": 1.0
                                                                    }
                                                                  ]
                            definition['States'][job+random_num] = definition['States'].pop(job)
                    else:

                        definition['States'][job]['Next'] = jobs[jobs.index(job) + 1] + random_num
                        if not job.startswith('['):
                            definition['States'][job]['Type'] = "Task"
                            definition['States'][job]['Resource'] = Resource
                            args_list = []
                            with open(args_path, "r") as args_file:
                                arg_line = args_file.readline()
                                while arg_line:
                                    args_list.append(arg_line.rstrip('\n'))
                                    arg_line = args_file.readline()
                            # 替换args.properties的参数
                            final_args_list = write_args(args_list, job_args)
                            Parameters['Step']['HadoopJarStep']['Args'][len(Parameters['Step']['HadoopJarStep']['Args'])
                                                                        :len(Parameters['Step']['HadoopJarStep']['Args'])] = final_args_list
                            definition['States'][job]['Parameters'] = Parameters
                            definition['States'][job]['ResultPath'] = "$.firstStep"
                            definition['States'][job]['Retry'] = [
                                {
                                    "ErrorEquals": ["States.ALL"],
                                    "IntervalSeconds": 1,
                                    "MaxAttempts": 3,
                                    "BackoffRate": 2.0
                                }
                            ]
                            definition['States'][job + random_num] = definition['States'].pop(job)
            return definition

        def create_parallel(states, job_name, s3_dag_path, dag_path, excution_name):
            if job_name.startswith('['):
                states[job_name] = {
                    "Type": "Parallel",
                    "Branches": []
                }
                # 取出[]中的并行的job
                if job_args["--g_month"]:
                    job_args["--g_month"] = str(int(job_args["--g_month"]) - 1)
                for parallel_job in job_name.strip('[]').replace(' ', '').split(','):
                    if job_args["--g_month"]:
                        job_args["--g_month"] = str(int(job_args["--g_month"]) + 1)
                    random_num = "_" + str(uuid.uuid4())
                    Parameters = {
                        "ClusterId.$": "$.clusterId",
                        "Step": {
                            "Name": "My EMR step",
                            "ActionOnFailure": "CONTINUE",
                            "HadoopJarStep": {
                                "Jar": "command-runner.jar",
                                "Args": ["spark-submit",
                                         "--deploy-mode", "cluster",
                                         "--py-files",
                                         "s3://ph-platform/2020-11-11/jobs/python/phcli/common/phcli-" + phcli_dv.CLI_CLIENT_VERSION + "-py3.8.egg," + s3_dag_path + parallel_job + "/phjob.py",
                                         s3_dag_path + parallel_job + "/phmain.py",
                                         "--owner", "default_owner",
                                         "--dag_name", s3_dag_path.split('/')[-2],
                                         "--run_id", s3_dag_path.split('/')[-2] + "_" + excution_name,
                                         "--job_full_name", parallel_job,
                                         "--job_id", "not_implementation",
                                         ]
                            }
                        }
                    }
                    step_tmp = {
                        "StartAt": "",
                        "States": {}
                    }
                    step_tmp.update({'StartAt': parallel_job + random_num})
                    # 遍历除第一行主分支的策略
                    for flow in flows:
                        # 如果有一行是以并行job开头，说明并行job还有延续，如果没有则States只有本身
                        if flow.startswith(parallel_job):
                            flow_jobs = []
                            # 根据 >> 进行分割
                            for flow_job_name in flow.split(' >> '):
                                flow_jobs.append(flow_job_name)
                                step_tmp['States'].update({flow_job_name: {}})
                                # 二次嵌套
                                # if flow_job_name.startswith('['):
                                #     flow_parallel_jobs = []
                                #     step_tmp['States'][flow_job_name] = {
                                #         "Type": "Parallel",
                                #         "Branches": []
                                #     }
                                #     for flow_parallel_job in flow_job_name.strip('[]').replace(' ', '').split(','):
                                #         flow_parallel_jobs.append(flow_parallel_job)
                                #         flow_step_tmp = {
                                #             "StartAt": "",
                                #             "States": {}
                                #         }
                                #         flow_step_tmp.update({'StartAt': flow_parallel_job})
                                #         for second_flow in flows:
                                #             if second_flow.startswith(flow_parallel_job):
                                #                 second_flow_jobs = []
                                #                 # 根据 >> 进行分割
                                #                 for second_flow_job_name in second_flow.split(' >> '):
                                #                     second_flow_jobs.append(second_flow_job_name)
                                #                     flow_step_tmp['States'].update({second_flow_job_name: {}})
                                #                 write_data(second_flow_jobs, flow_step_tmp, s3_dag_path, dag_path, excution_name)
                                #         if flow_step_tmp['States'] == {}:
                                #             flow_step_tmp['States'].update({flow_step_tmp['StartAt']: {'End': True}})
                                #             flow_step_tmp['States'][flow_step_tmp['StartAt']]['Type'] = "Task"
                                #             flow_step_tmp['States'][flow_step_tmp['StartAt']]['Resource'] = Resource
                                #             flow_step_tmp['States'][flow_step_tmp['StartAt']]['Parameters'] = Parameters
                                #             flow_step_tmp['States'][flow_step_tmp['StartAt']][
                                #                 'ResultPath'] = "$.firstStep"
                                #         step_tmp['States'][flow_job_name]['Branches'].append(flow_step_tmp)
                                #     write_data(flow_parallel_jobs, flow_step_tmp, s3_dag_path, dag_path, excution_name)
                            write_data(flow_jobs, step_tmp, s3_dag_path, dag_path, excution_name, job_args, random_num)

                    # 判断States是否为空 如果为空 说明没有其他延续 则本身作为States
                    if step_tmp['States'] == {}:
                        step_tmp['States'].update({step_tmp['StartAt']: {'End': True}})
                        step_tmp['States'][step_tmp['StartAt']]['Type'] = "Task"
                        step_tmp['States'][step_tmp['StartAt']]['Resource'] = Resource
                        step_tmp['States'][step_tmp['StartAt']]['Parameters'] = Parameters
                        step_tmp['States'][step_tmp['StartAt']]['ResultPath'] = "$.firstStep"
                    states[job_name]['Branches'].append(step_tmp)

            return states

        if self.strategy == "v2":
            for key in os.listdir(self.dag_path):
                if os.path.isfile(self.dag_path + key):
                    self.phs3.upload(
                        file=self.dag_path+key,
                        bucket_name=dv.DAGS_S3_BUCKET,
                        object_name=dv.DAGS_S3_PREV_PATH + key
                    )
                else:
                    self.phs3.upload_dir(
                        dir=self.dag_path+key,
                        bucket_name=dv.TEMPLATE_BUCKET,
                        s3_dir=dv.CLI_VERSION + dv.DAGS_S3_PHJOBS_PATH + self.name + "/" + key
                    )

        if self.strategy == "v3":
            job_args = eval(self.job_args)
            excution_name = self.name + "_" + time.strftime("%Y-%m-%d_%H-%M-%S", time.localtime())
            for key in os.listdir(self.dag_path):
                if os.path.isfile(self.dag_path + key):
                    self.phs3.upload(
                        file=self.dag_path+key,
                        bucket_name=dv.DAGS_S3_BUCKET,
                        object_name=dv.DAGS_S3_PREV_PATH + key
                    )
                else:
                    self.phs3.upload_dir(
                        dir=self.dag_path+key,
                        bucket_name=dv.TEMPLATE_BUCKET,
                        s3_dir=dv.CLI_VERSION + dv.DAGS_S3_PHJOBS_PATH + self.name + "/" + key
                    )
                s3_dag_path = "s3://" + dv.TEMPLATE_BUCKET + "/" + dv.CLI_VERSION + dv.DAGS_S3_PHJOBS_PATH + self.name + "/"
                dag_path = self.dag_path
                if os.path.isfile(self.dag_path + key):

                    Resource = "arn:aws-cn:states:::elasticmapreduce:addStep.sync"
                    # 如果是 file 则为 dag 产生的 py文件， 判断文件最后一行设置的策略 创建step流程模板
                    # 策略的每一行，存放到每一个列表
                    flows = []
                    # 一行中每一个job的名称
                    jobs = []
                    # 状态机的所有states
                    states = {}
                    with open(self.dag_path + key, "r") as dag_file:
                        line = dag_file.readline()
                        while line:
                            while ">>" in line:
                                flows.append(line.rstrip('\n'))
                                break
                            line = dag_file.readline()
                    for job_name in flows[0].split(' >> '):
                        states[job_name] = {}
                        jobs.append(job_name)
                        # 若果是以"[" 开头的job 对并行的job进行操作
                        states = create_parallel(states, job_name, s3_dag_path, dag_path, excution_name)
                        if len(job_name) > 60:
                            states[job_name[:59]] = states.pop(job_name)
                            jobs = [job_name[:59] if i == job_name else i for i in jobs]
                    definition = {
                        "StartAt": "",
                        "States": {}
                    }
                    definition['StartAt'] = list(states.keys())[0]
                    definition['States'] = states
                    write_data(jobs, definition, s3_dag_path, dag_path, excution_name, job_args)

                    create_definition = json.dumps(definition)

                    step_client = boto3.client('stepfunctions')
                    response = step_client.create_state_machine(
                        name=self.name,
                        definition=create_definition,
                        roleArn=os.getenv("DEFAULT_ROLE_ARN"),
                        type=dv.DEFAULT_MACHINE_TYPE,
                    )

                    ssm_client = boto3.client('ssm')
                    ssm_response = ssm_client.get_parameter(
                        Name='cluster_id'
                    )

                    machine_input = {
                        'clusterId': ssm_response['Parameter']['Value']
                    }
                    step_client.start_execution(
                        stateMachineArn=response['stateMachineArn'],
                        name=excution_name,
                        input=json.dumps(machine_input)
                    )

    def recall(self, **kwargs):
        """
        默认的召回过程
        """
        self.logger.debug('maxauto 默认的 recall 实现')
        self.logger.debug(self.__dict__)

        self.phs3.delete_dir(dv.TEMPLATE_BUCKET, dv.CLI_VERSION + dv.DAGS_S3_PHJOBS_PATH + self.name)
        self.phs3.delete_dir(dv.DAGS_S3_BUCKET, dv.DAGS_S3_PREV_PATH + self.get_dag_py_file_name(self.name))

    def online_run(self, **kwargs):
        """
        默认的 online_run 过程
        """
        self.logger.debug('maxauto 默认的 online_run 实现')
        self.logger.debug(self.__dict__)
        def write_data(definition, s3_dag_path, job_name, excution_name, job_args_name_list):
            Resource = "arn:aws-cn:states:::elasticmapreduce:addStep.sync"
            Parameters = {
                "ClusterId.$": "$.clusterId",
                "Step": {
                    "Name": "My EMR step",
                    "ActionOnFailure": "CONTINUE",
                    "HadoopJarStep": {
                        "Jar": "command-runner.jar",
                        "Args": ["spark-submit",
                                 "--deploy-mode", "cluster",
                                 "--conf", "spark.driver.cores=1",
                                 "--conf", "spark.driver.memory=1g",
                                 "--conf", "spark.executor.cores=1",
                                 "--conf", "spark.executor.memory=1g",
                                 "--conf", "spark.executor.instances=1",
                                 "--conf", "spark.hadoop.fs.s3a.impl=org.apache.hadoop.fs.s3a.S3AFileSystem",
                                 "--conf", "spark.hadoop.fs.s3a.access.key=AKIAWPBDTVEANKEW2XNC",
                                 "--conf", "spark.hadoop.fs.s3a.secret.key=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",
                                 "--conf", "spark.hadoop.fs.s3a.endpoint=s3.cn-northwest-1.amazonaws.com.cn",
                                 "--py-files",
                                 "s3://ph-platform/2020-11-11/jobs/python/phcli/common/phcli-" + phcli_dv.CLI_CLIENT_VERSION + "-py3.8.egg," + s3_dag_path + "phjob.py",
                                 s3_dag_path + "phmain.py",
                                 "--owner", "default_owner",
                                 "--dag_name", s3_dag_path.split('/')[-2],
                                 "--run_id", s3_dag_path.split('/')[-2] + "_" + excution_name,
                                 "--job_full_name", job_name,
                                 "--job_id", "not_implementation"
                                 ]
                    }
                }
            }
            definition['States'][job_name]['End'] = True
            definition['States'][job_name]['Type'] = "Task"
            definition['States'][job_name]['Resource'] = Resource
            Parameters['Step']['HadoopJarStep']['Args'][len(Parameters['Step']['HadoopJarStep']['Args'])
                                                        :len(
                Parameters['Step']['HadoopJarStep']['Args'])] = job_args_name_list
            definition['States'][job_name]['Parameters'] = Parameters
            definition['States'][job_name]['ResultPath'] = "$.firstStep"
            definition['States'][job_name]['Retry'] = [
                {
                    "ErrorEquals": ["States.ALL"],
                    "IntervalSeconds": 1,
                    "MaxAttempts": 1,
                    "BackoffRate": 1.0
                }
            ]
            definition['States'][job_name] = definition['States'].pop(job_name)

        def write_args(args_list, job_args):
            keys = []
            values = []
            # 获取list中的参数并 生成一个dict
            for arg in args_list:
                if args_list.index(arg) % 2 == 0:
                    keys.append(arg)
                elif args_list.index(arg) % 2 == 1:
                    values.append(arg)
            args = zip(keys, values)

            args_dict = dict(args)
            # 获取生成dict中的参数, 将传进来的dict进行替换
            for key in args_dict.keys():
                if key.lstrip('--') in job_args.keys():
                    args_dict[key] = job_args[key.lstrip('--')]

            new_key = []
            for args_key in args_dict.keys():
                new_key.append(args_key.lstrip('--'))
            new_args = dict(zip(new_key, args_dict.values()))
            return new_args

        def ast_parse(string):
            """
            解析json
            :param string: json 字符串
            :return: dict
            """
            ast_dict = {}
            if string != "":
                ast_dict = ast.literal_eval(string.replace(" ", ""))
                for k, v in ast_dict.items():
                    if isinstance(v, str) and v.startswith('{') and v.endswith('}'):
                        ast_dict[k] = ast.literal_eval(v)
            return ast_dict

        # airflow 运行
        self.context = ast_parse(self.context)
        self.args = ast_parse(self.args)

        self.s3_job_path = dv.DAGS_S3_PHJOBS_PATH + self.dag_name + "/" + self.job_full_name
        self.submit_prefix = "s3a://" + dv.TEMPLATE_BUCKET + "/" + dv.CLI_VERSION + self.s3_job_path + "/"

        stream = self.phs3.open_object(dv.TEMPLATE_BUCKET, dv.CLI_VERSION + self.s3_job_path + "/phconf.yaml")
        config = PhYAMLConfig()
        # config.load_yaml(stream)
        # self.runtime = config.spec.containers.runtime
        # self.command = config.spec.containers.command
        # self.timeout = config.spec.containers.timeout
        #
        # runtime_inst = self.table_driver_runtime_inst(self.runtime)
        # runtime_inst(**self.__dict__).online_run()


        # step functions 运行
        s3_dag_path = "s3://" + dv.TEMPLATE_BUCKET + "/" + dv.CLI_VERSION + self.s3_job_path + "/"
        job_name = self.name
        definition = {
            "StartAt": "",
            "States": {}
        }
        states = {}
        states[job_name] = {}
        definition['StartAt'] = job_name
        definition['States'] = states
        random_num = "_" + str(uuid.uuid4())
        job_args_name = 'args' + random_num
        # 将 job_args_name 写成list
        job_args_name_list = ['--job_args_name',job_args_name]
        excution_name = self.name + "_" + time.strftime("%Y-%m-%d_%H-%M-%S", time.localtime())
        job_args = self.args

        args_list = self.phs3.open_object_by_lines(dv.TEMPLATE_BUCKET,
                                                   dv.CLI_VERSION + self.s3_job_path + "/args.properties")
        new_args_list = []
        for arg in args_list:
            if arg.startswith('s3a'):
                arg = arg.replace('s3a:', 's3:')
            new_args_list.append(arg)
        args_dict = write_args(new_args_list, job_args)
        write_data(definition, s3_dag_path, job_name, excution_name, job_args_name_list)

        create_definition = json.dumps(definition)

        step_client = boto3.client('stepfunctions')
        # 创建状态机

        step_create_response = step_client.create_state_machine(
            name=self.name + random_num,
            definition=create_definition,
            roleArn=dv.DEFAULT_ROLE_ARN,
            type=dv.DEFAULT_MACHINE_TYPE,
        )
        # 获取cluster_id
        ssm_client = boto3.client('ssm')
        ssm_response = ssm_client.get_parameter(
            Name='cluster_id'
        )
        machine_input = {
            'clusterId': ssm_response['Parameter']['Value']
        }
        ssm_response = ssm_client.get_parameter(
            Name='cluster_id'
        )

        # 將kwargs写入ssm
        ssm_client.put_parameter(
            Name=job_args_name,
            Value=str(args_dict),
            Type='String'
        )

        # 启动状态机
        start_response = step_client.start_execution(
            stateMachineArn=step_create_response['stateMachineArn'],
            name=excution_name,
            input=json.dumps(machine_input)
        )

        # 获取状态机的执行状态 直到成功退出
        execution_response = step_client.list_executions(
            stateMachineArn=step_create_response['stateMachineArn'],
        )

        while execution_response:
            time.sleep(60)
            if len(execution_response['executions']) == 0:
                execution_response = step_client.list_executions(
                    stateMachineArn=step_create_response['stateMachineArn'],
                )
                continue

            execution_response = step_client.list_executions(
                stateMachineArn=step_create_response['stateMachineArn'],
            )
            execution_status = execution_response['executions'][0]['status']
            if execution_status == 'SUCCEEDED':
                step_client.delete_state_machine(
                    stateMachineArn=step_create_response['stateMachineArn']
                )
                break
            if execution_status == 'FAILED':
                step_client.delete_state_machine(
                    stateMachineArn=step_create_response['stateMachineArn']
                )
                raise Exception("Job运行失败")
                break

        return 0


    def status(self, **kwargs):
        """
        默认的查看运行状态
        """
        self.logger.debug('maxauto 默认的 status 实现')
        self.logger.debug(self.__dict__)



import uuid
from datetime import datetime
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy import Column, String, DateTime, Float


Base = declarative_base()


class Asset(Base):
    __tablename__ = 'asset'

    id = Column(String, primary_key=True, default=lambda: str(uuid.uuid4()))
    name = Column(String)
    owner = Column(String)
    extension = Column(String, default='parquet')
    block = Column(String, default='{}')
    size = Column(Float, default=-1)
    providers = Column(String, default='{}')
    markets = Column(String, default='{}')
    molecules = Column(String, default='{}')
    dateCover = Column(String, default='{}')
    geoCover = Column(String, default='{}')
    labels = Column(String, default='{}')
    created = Column(DateTime, default=datetime.now())
    modified = Column(DateTime, default=datetime.now())
    description = Column(String)
    type = Column(String, default='file')
    source = Column(String)
    version = Column(String, default='0.0.1')
    isNewVersion = Column(String, default='t')
    accessibility = Column(String, default='w')
    shared = Column(String)
    partners = Column(String)

    def __repr__(self):
        return str(self.__dict__)


if __name__ == '__main__':
    asset = Asset(id="id", name="name", owner="owner", source="source")
    print(asset)

    import base64
    from phcli.ph_db.ph_pg import PhPg

    pg = PhPg(
        base64.b64decode('cGgtZGItbGFtYmRhLmNuZ2sxamV1cm1udi5yZHMuY24tbm9ydGh3ZXN0LTEuYW1hem9uYXdzLmNvbS5jbgo=').decode('utf8')[:-1],
        base64.b64decode('NTQzMgo=').decode('utf8')[:-1],
        base64.b64decode('cGhhcmJlcnMK').decode('utf8')[:-1],
        base64.b64decode('QWJjZGUxOTYxMjUK').decode('utf8')[:-1],
        db=base64.b64decode('cGhlbnRyeQo=').decode('utf8')[:-1],
    )
    pg.insert(asset)
    pg.commit()



import boto3
from phcli.ph_aws.ph_sts import PhSts


def test_ph_sts_assume_role():
    credentials = PhSts().assume_role('arn:aws-cn:iam::444603803904:role/Ph-Cli-Lmd', 'Ph-Cli-Lmd').get_cred()
    assert credentials is not None
    s3_client = boto3.client('s3', **credentials)
    assert s3_client.list_buckets() is not None



# -*- coding: utf-8 -*-

import os
from phcli.ph_data_clean.script.s3_traverse import LOCAL_CACHE_DIR
from phcli.ph_data_clean.util.yaml_utils import load_by_file, override_to_file


def simple_file_duplication(file):
    dup_lst = load_by_file(file)
    simple_lst = [eval(t) for t in set([str(d) for d in dup_lst])]
    override_to_file(simple_lst, file)


if __name__ == '__main__':
    """
    清理缓存文件中的重复数据
    """
    for sub in os.listdir(LOCAL_CACHE_DIR):
        file = LOCAL_CACHE_DIR + sub
        simple_file_duplication(file)



# -*- coding: utf-8 -*-

from phcli.ph_errs.ph_err import PhException
from phcli.ph_data_clean.clean.cpa_gyc_data_clean import CpaGycDataClean
from phcli.ph_data_clean.clean.chc_data_clean import ChcDataClean
from phcli.ph_data_clean.clean.product_data_clean import ProductDataClean
from phcli.ph_data_clean.clean.universe_data_clean import UniverseDataClean


class CleanerFactory(object):
    """
    清洗算法的生成工厂
    """

    all_clean = {
        ('CPA', 'GYC', 'CPA&GYC', 'GYC&CPA', 'CPA&PTI&DDD&HH'): CpaGycDataClean,
        ('CHC',): ChcDataClean,
        ('universe',): UniverseDataClean,
        ('product',): ProductDataClean,
    }

    def get_specific_cleaner(self, source, company=''):
        """
        根据源和公司获取特定的清洗算法

        :param source: 清洗的元数据类型
        :param company: 清洗的公司名称

        :return: [DataClean] 特定清洗算法


        """

        finded = [clean for clean in self.all_clean.items()
                  if source.lower() in [item.lower() for item in clean[0]]]

        if len(finded) == 1:
            return finded[0][1]()
        elif len(finded) > 1:
            raise PhException("Find more Cleaner" + str(finded))
        else:
            raise PhException("Not find Cleaner, source=%s, company=%s" % (source, company))




# -*- coding: utf-8 -*-

from abc import abstractmethod
import os

from phcli.ph_logs.ph_logs import phlogger
from phcli.ph_errs.ph_err import PhException


class LambdaRuntime(object):
    """
    封装 AWS Lambda 各个运行时的的常规操作和抽象方法
    """

    _package_root = ".package"
    _package_cmds = []

    @abstractmethod
    def pkg_layer(self, data):
        pass

    @abstractmethod
    def pkg_code(self, data):
        pass

    def package(self, package_name):
        """
        执行打包逻辑
        :param package_name 打包的名称
        """

        if not isinstance(self._package_cmds, list):
            raise PhException('Expected an list')

        self._package_cmds.extend([
            "mv package.zip %s" % package_name,
        ])

        try:
            for cmd in self._package_cmds:
                phlogger.info("正在执行: " + cmd + " ")
                os.system(cmd)
        except Exception as ex:
            phlogger.error(ex)



# -*- coding: utf-8 -*-

import copy


class DataClean(object):
    """
    清洗方法的父类
    """

    process = []

    def cleaning_process(self, *args, **kwargs):
        runtime_class_name = self.__class__.__name__

        func_result = None
        for func in self.process:
            # 类方法调用方式，需要传递 self
            if runtime_class_name in str(func):
                func_result = func(self, *args, **kwargs)
            # 函数调用方式
            else:
                func_result = func(*args, **kwargs)

            if func_result:
                kwargs[func.__name__] = copy.deepcopy(func_result)
                kwargs['prev'] = func_result
        return func_result



import io
import os
import boto3
import pandas as pd
from pyspark.sql import SparkSession
import pyspark.sql.functions as sf

access_key = "AKIAWPBDTVEAJ6CCFVCP"
secret_key = "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"

SOURCE_BUCKET = 'ph-origin-files'
SOURCE_PATH = 'prod/Product standardization master data-A-S-0827.xlsx'
TARGET_BUCKET = 'ph-stream'
TARGET_PATH = 'common/public/prod/0.0.15'


s3_client = boto3.client('s3', aws_access_key_id=access_key, aws_secret_access_key=secret_key)
object_file = s3_client.get_object(Bucket=SOURCE_BUCKET, Key=SOURCE_PATH)
data = object_file['Body'].read()
pd_df = pd.read_excel(io.BytesIO(data), encoding='utf-8')

os.environ["PYSPARK_PYTHON"] = "python3"
spark = SparkSession.builder \
    .master("yarn") \
    .appName("data cube cal measures") \
    .config("spark.driver.memory", "1g") \
    .config("spark.executor.cores", "2") \
    .config("spark.executor.instance", "4") \
    .config("spark.executor.memory", "2g") \
    .config('spark.sql.codegen.wholeStage', False) \
    .getOrCreate()

if access_key is not None:
    spark._jsc.hadoopConfiguration().set("fs.s3a.access.key", access_key)
    spark._jsc.hadoopConfiguration().set("fs.s3a.secret.key", secret_key)
    spark._jsc.hadoopConfiguration().set("fs.s3a.impl","org.apache.hadoop.fs.s3a.S3AFileSystem")
    spark._jsc.hadoopConfiguration().set("com.amazonaws.services.s3.enableV4", "true")
    # spark._jsc.hadoopConfiguration().set("fs.s3a.aws.credentials.provider","org.apache.hadoop.fs.s3a.BasicAWSCredentialsProvider")
    spark._jsc.hadoopConfiguration().set("fs.s3a.endpoint", "s3.cn-northwest-1.amazonaws.com.cn")

sdf = spark.createDataFrame(pd_df.astype(str)).drop('Unnamed: 39') \
    .withColumn('COMPANY', sf.lit('product')) \
    .withColumn('SOURCE', sf.lit('product')) \
    .withColumn('TAG', sf.lit(SOURCE_PATH.split('/')[-1])) \
    .withColumn('version', sf.lit('0.0.15'))
sdf.show()
save_path = "s3a://%s/%s" % (TARGET_BUCKET, TARGET_PATH)
sdf.write.format("parquet").mode("overwrite").save(save_path)



# -*- coding: utf-8 -*-

import boto3
import yaml
import base64

from phcli.ph_aws.ph_sts import PhSts
from phcli.ph_aws.ph_s3 import PhS3
from phcli.ph_lmd.model.aws_operator import AWSOperator
from phcli.ph_lmd.model.ph_lambda import PhLambda
from phcli.ph_lmd import define_value as dv


class PhGateway(AWSOperator):
    """
    lambda 的 API Gateway 代理
    """

    phsts = PhSts().assume_role(
        base64.b64decode(dv.ASSUME_ROLE_ARN).decode(),
        dv.ASSUME_ROLE_EXTERNAL_ID,
    )
    phs3 = PhS3(phsts=phsts)

    def __put_resource_by_template(self, rest_api_id, project_name, paths, lambda_arn, role_arn):
        api_gateway_client = boto3.client('apigateway', **self.phsts.get_cred())

        def put_integration(rest_api_id, resource_id, method, lambda_arn, role_arn):
            response = api_gateway_client.put_integration(
                restApiId=rest_api_id,
                resourceId=resource_id,
                httpMethod=method.upper(),
                type='AWS_PROXY',
                integrationHttpMethod="POST",
                uri='arn:aws-cn:apigateway:cn-northwest-1:lambda:path/2015-03-31/functions/' + lambda_arn + '/invocations',
                # connectionType='INTERNET'|'VPC_LINK',
                # connectionId='string',
                credentials=role_arn,
                # requestParameters={
                #     'string': 'string'
                # },
                # requestTemplates={
                #     'string': 'string'
                # },
                # passthroughBehavior='string',
                # cacheNamespace='string',
                # cacheKeyParameters=[
                #     'string',
                # ],
                # contentHandling='CONVERT_TO_BINARY'|'CONVERT_TO_TEXT',
                # timeoutInMillis=123,
                # tlsConfig={
                #     'insecureSkipVerification': True|False
                # }
            )
            return response

        path_id_dict = {}
        for reso in api_gateway_client.get_resources(restApiId=rest_api_id, limit=9999)["items"]:
            path_id_dict[reso["path"]] = reso["id"]

        for path, methods in sorted(paths.items()):
            path = path.replace("{project_name}", project_name)
            prefix = []

            # 添加资源
            for sub_path in path.split("/")[1:]:
                parent_id = path_id_dict.get("/" + "/".join(prefix))
                prefix.append(sub_path)
                path = "/" + "/".join(prefix)
                if path not in path_id_dict.keys():
                    resource_id = api_gateway_client.create_resource(
                        restApiId=rest_api_id,
                        parentId=parent_id,
                        pathPart=sub_path,
                    )["id"]
                    path_id_dict[path] = resource_id

            # 为资源添加方法并集成 lambda
            for method, params in methods.items():
                api_gateway_client.put_method(
                    restApiId=rest_api_id,
                    resourceId=path_id_dict.get(path),
                    httpMethod=method,
                    authorizationType='AWS_IAM',
                    # authorizerId='string',
                    # apiKeyRequired=True|False,
                    # operationName='string',
                    requestParameters={
                        'method.request.header.Accept': True,
                        'method.request.header.Content-Type': True,
                    },
                    # requestModels={
                    #     'string': 'string'
                    # },
                    # requestValidatorId='string',
                    # authorizationScopes=[
                    #     'string',
                    # ]
                )

                put_integration(
                    rest_api_id=rest_api_id,
                    resource_id=path_id_dict.get(path),
                    method=method,
                    lambda_arn=lambda_arn,
                    role_arn=role_arn
                )

    def __create_deployment(self, rest_api_id, version, gateway_desc):
        api_gateway_client = boto3.client('apigateway', **self.phsts.get_cred())
        response = api_gateway_client.create_deployment(
            restApiId=rest_api_id,
            stageName=version,
            stageDescription=gateway_desc,
            description=gateway_desc,
            # cacheClusterEnabled=True|False,
            # cacheClusterSize='0.5'|'1.6'|'6.1'|'13.5'|'28.4'|'58.2'|'118'|'237',
            # variables={
            #     'string': 'string'
            # },
            # canarySettings={
            #     'percentTraffic': 123.0,
            #     'stageVariableOverrides': {
            #         'string': 'string'
            #     },
            #     'useStageCache': True|False
            # },
            # tracingEnabled=True|False
        )
        return response

    def package(self, data):
        """
        API Gateway 代理 不可打包
        """
        return self.package.__doc__

    def create(self, data):
        """
        创建 lambda 的 API Gateway 的根资源
        :param data:
            :arg name: API Gateway 的根资源名称
            :arg rest_api_id: rest API Gateway 的 ID
            :arg api_template: API Gateway 定义的符合 OpenAPI 规范的模板文档位置
                        可以是本地（file/ph_lmd/jsonapi-openapi-template.yaml，会先被传到 S3）或
                        s3 上的文件（s3://ph-platform/2020-08-10/template/python/phcli/lmd/jsonapi-openapi-template.yaml）
            :arg lambda_name: API Gateway 调用的 Lambda 函数
            :arg alias_version: 代理的 lambda function 的别名版本
        """
        rest_api_id = data["rest_api_id"]
        project_name = data["name"]

        bucket_name, object_name = self.phs3.sync_file_local_to_s3(
            data["api_template"],
            bucket_name=data.get("bucket", dv.DEFAULT_BUCKET),
            dir_name=dv.CLI_VERSION + dv.DEFAULT_TEMPLATE_DIR,
        )
        buf = self.phs3.open_object(bucket_name, object_name)
        gateway_conf = yaml.safe_load(buf)

        self.__put_resource_by_template(
            rest_api_id=rest_api_id,
            project_name=project_name,
            paths=gateway_conf["paths"],
            lambda_arn=PhLambda().get({"name": data["lambda_name"]})["Configuration"]["FunctionArn"] + ":" + data["alias_version"],
            role_arn=base64.b64decode(dv.ASSUME_ROLE_ARN).decode(),
        )

        return rest_api_id + "/" + project_name + " 生成成功"

    def lists(self, data):
        """
        获取所有 lambda 的 API Gateway 代理实例
        :param data:
            :arg rest_api_id: rest API Gateway 的 ID
        """
        api_gateway_client = boto3.client('apigateway', **self.phsts.get_cred())

        if "rest_api_id" in data.keys():
            response = api_gateway_client.get_rest_api(
                restApiId=data["rest_api_id"]
            )
        else:
            response = api_gateway_client.get_rest_apis(
                # position='string',
                limit=9999,
            )
        return response

    def get(self, data):
        """
        获取指定 lambda 的 API Gateway 根资源名称
        :param data:
            :arg name: API Gateway 的名称【可能会查出多个】
            :arg rest_api_id: rest API Gateway 的 ID
        """
        api_gateway_client = boto3.client('apigateway', **self.phsts.get_cred())

        response = api_gateway_client.get_resources(
            restApiId=data["rest_api_id"],
            # position='string',
            limit=9999,
            # embed=[
            #     'string',
            # ]
        )

        items = []
        for api in response["items"]:
            if data["name"] in api["path"]:
                items.append(api)

        if not len(items):
            return {}

        response["items"] = items
        return response

    def update(self, data):
        """
        更新指定 lambda 的 API Gateway 的根资源, 操作等于是 create
        :param data:
            :arg name: API Gateway 的根资源名称
            :arg rest_api_id: rest API Gateway 的 ID
            :arg api_template: API Gateway 定义的符合 OpenAPI 规范的模板文档位置
                        可以是本地（file/ph_lmd/jsonapi-openapi-template.yaml，会先被传到 S3）或
                        s3 上的文件（s3://ph-platform/2020-08-10/template/python/phcli/lmd/jsonapi-openapi-template.yaml）
            :arg lambda_name: API Gateway 调用的 Lambda 函数
            :arg alias_version: 代理的 lambda function 的别名版本
        """
        return self.create(data)

    def apply(self, data):
        """
        创建或更新 lambda 的 API Gateway 的根资源, 操作等于是 create
        :param data:
            :arg name: API Gateway 的根资源名称
            :arg rest_api_id: rest API Gateway 的 ID
            :arg api_template: API Gateway 定义的符合 OpenAPI 规范的模板文档位置
                        可以是本地（file/ph_lmd/jsonapi-openapi-template.yaml，会先被传到 S3）或
                        s3 上的文件（s3://ph-platform/2020-08-10/template/python/phcli/lmd/jsonapi-openapi-template.yaml）
            :arg lambda_name: API Gateway 调用的 Lambda 函数
            :arg alias_version: 代理的 lambda function 的别名版本
        """
        if self.get(data) == {}:
            return self.create(data)
        else:
            return self.update(data)

    def stop(self, data):
        """
        API Gateway 代理实例不可停止
        """
        return self.stop.__doc__

    def start(self, data):
        """
        API Gateway 代理实例不可启动
        """
        return self.start.__doc__

    def delete(self, data):
        """
        删除 API Gateway 中的指定根资源
        :param data:
            :arg name: API Gateway 根资源名称
            :arg rest_api_id: rest API Gateway 的 ID
        """
        api_gateway_client = boto3.client('apigateway', **self.phsts.get_cred())

        response = {}
        for item in self.get(data)["items"]:
            if "/" + data["name"] == item["path"]:
                response = api_gateway_client.delete_resource(
                    restApiId=data['rest_api_id'],
                    resourceId=item["id"]
                )
        return response



import uuid
from datetime import datetime
from sqlalchemy.orm import relationship
from sqlalchemy import Column, String, ForeignKey
from sqlalchemy.ext.declarative import declarative_base


Base = declarative_base()


class Partner(Base):
    __tablename__ = 'partner'

    id = Column(String, primary_key=True, default=lambda: str(uuid.uuid4()))
    name = Column(String)
    address = Column(String)
    phoneNumber = Column(String)
    web = Column(String)
    employee = Column(String, default='{}')
    accounts = relationship('Account', back_populates="partner")
    created = Column(String, default=datetime.now())
    modified = Column(String, default=datetime.now())

    def __repr__(self):
        return str(self.__dict__)


class Account(Base):
    __tablename__ = 'account'

    id = Column(String, primary_key=True, default=lambda: str(uuid.uuid4()))
    name = Column(String)
    wechatOpenId = Column(String)
    password = Column(String)
    phoneNumber = Column(String)
    defaultRole = Column(String, ForeignKey('role.id'))
    role = relationship('Role', back_populates='accounts')
    email = Column(String)
    employer = Column(String, ForeignKey('partner.id'))
    partner = relationship('Partner', back_populates='accounts')
    created = Column(String, default=datetime.now())
    modified = Column(String, default=datetime.now())
    firstName = Column(String)
    lastName = Column(String)

    def __repr__(self):
        return str(self.__dict__)


class Role(Base):
    __tablename__ = 'role'

    id = Column(String, primary_key=True, default=lambda: str(uuid.uuid4()))
    name = Column(String)
    accountRole = Column(String, default='{}')
    accounts = relationship('Account', back_populates="role")
    description = Column(String)
    scope = Column(String, default='{}')
    created = Column(String, default=datetime.now())
    modified = Column(String, default=datetime.now())

    def __repr__(self):
        return str(self.__dict__)


class Scope(Base):
    __tablename__ = 'scope'

    id = Column(String, primary_key=True, default=lambda: str(uuid.uuid4()))
    name = Column(String)
    description = Column(String)
    scopePolicy = Column(String)
    owner = Column(String, default='{}')
    created = Column(String, default=datetime.now())
    modified = Column(String, default=datetime.now())

    def __repr__(self):
        return str(self.__dict__)


if __name__ == '__main__':
    import base64
    from phcli.ph_db.ph_pg import PhPg

    pg = PhPg(
        base64.b64decode('cGgtZGItbGFtYmRhLmNuZ2sxamV1cm1udi5yZHMuY24tbm9ydGh3ZXN0LTEuYW1hem9uYXdzLmNvbS5jbgo=').decode('utf8')[:-1],
        base64.b64decode('NTQzMgo=').decode('utf8')[:-1],
        base64.b64decode('cGhhcmJlcnMK').decode('utf8')[:-1],
        base64.b64decode('QWJjZGUxOTYxMjUK').decode('utf8')[:-1],
        db=base64.b64decode('cGhjb21tb24K').decode('utf8')[:-1],
    )

    pn = Partner(name="test", address="test", phoneNumber="test", web="test")
    pn = pg.insert(pn)
    pn = pg.query(pn)
    pn = pn[0] if pn else None

    ac = Account(name="test", wechatOpenId="test", password="test", phoneNumber="test",
                 email='test', firstName='test', lastName='test')
    ac.partner = pn
    ac = pg.insert(ac)
    pg.update(Partner(id=pn.id, employee=pn.employee+[ac.id]))

    ac2 = Account(name="test2", wechatOpenId="test2", password="test2", phoneNumber="test2",
                  email='test2', firstName='test2', lastName='test2')
    ac2.partner = pn
    ac2 = pg.insert(ac2)
    pg.update(Partner(id=pn.id, employee=pn.employee+[ac2.id]))

    pg.commit()



